// Copyright (c) 2009 Yahoo! Inc. 
// All Rights Reserved
// 
// this service provides anonymous install statistics for FoxyTunes
// which helps us understand how many people are installing FoxyTunes
// the data is only used for stats in an aggregated form
// no personal information is ever disclosed 
// please refer to our Privacy Policy for more details
function FoxyTunesAnonymousInstallStatsService() {
    this._prefService = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);
    this._prefBranch  = this._prefService.getBranch("foxytunes.");
    this._foxytunesAnonymousUsageStatsService = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesAnonymousUsageStatsService;1"]
                            .getService().wrappedJSObject;
    this._toolbarIsActivated = null;
    this._toolbarActivePref = 'stats.toolbar-is-active';

    try {
       Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
       getService(Components.interfaces.mozIJSSubScriptLoader).
       loadSubScript('chrome://foxytunes/content/utils.js');
    } catch (e) {};

    this._baseURL = 'http://a.analytics.yahoo.com/p.pl';
    this._projectID = '1000944194111';

    this.wrappedJSObject = this;
}

FoxyTunesAnonymousInstallStatsService.prototype = {
    classID : Components.ID("{1C8BFC60-8E56-11DD-8175-657D56D89593}"),

    toolbarActive: function() {
        if (!this._IsToolbarActive()) {
            this._prefBranch.setBoolPref(this._toolbarActivePref, true);
            this._toolbarIsActivated = true;
            var daysSinceInstall = null;
            try {
                var currentTime = new Date();
                var installTime = this._prefBranch.getCharPref('install_time');
                var daysSinceInstall = Math.round((currentTime - installTime) / 86400000);
            } catch (e) {}
            if (daysSinceInstall == null) return;
            var params = 'x=4&cf6=' + daysSinceInstall;
            this._sendAnonymousStats('activation', params);
        } else {
            this._foxytunesAnonymousUsageStatsService.showOptInIfNeeded();
        }
    },

    toolbarInstalled: function() {
        // Reset activation on now installs
        this._prefBranch.setBoolPref(this._toolbarActivePref, false);
        this._sendAnonymousStats('install', 'x=2&cf7=install');
    },

    toolbarUpdated: function(foxytunesPrevVersion) {
        this._sendAnonymousStats('update', 'x=3&cf5=' + foxytunesPrevVersion);
    },

    toolbarUninstalled: function() {
        this._sendAnonymousStats('uninstall', 'x=5');
    },

    // private functions
    _IsToolbarActive: function() {
        if (this._toolbarIsActivated == null) {
            this._toolbarIsActivated = false;
            try {
                this._toolbarIsActivated = this._prefBranch.getBoolPref(this._toolbarActivePref);           
            } catch (e) {}
        }
        return this._toolbarIsActivated;
    },

    // Used in order to prevent the garbage collection
    _onReadyStateChanged: function (){},

    _sendAnonymousStats: function(type, customParams) {
       var app = Components.classes["@mozilla.org/xre/app-info;1"]
                   .getService(Components.interfaces.nsIXULAppInfo)
                   .QueryInterface(Components.interfaces.nsIXULRuntime);

       var ftversion = '-';
       try {
        ftversion = this._prefBranch.getCharPref('version');
       } catch (e) {};

       var locale = '-';
       try {
           locale = this._prefService.getBranch('general.').getCharPref('useragent.locale');
       } catch (e) {};

        var appName = foxytunesGetAppName();

        var url = this._baseURL + '?a=' + this._projectID;
        url += '&b=' + type;
        url += '&f=' + 'http%3A%2F%2F' + type + '%2F';
        url += '&c=' +  ftversion;
        url += '&cf1=' + appName;
        url += '&cf2=' + app.version;
        url += '&cf4=' + locale;
        url += '&l=true';
        if (customParams) {
            url += '&' + customParams;    
       }

      //  dump("Sending stat: " + url + "\n");

       var xmlHTTPrequest = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].
                                    createInstance(Components.interfaces.nsIXMLHttpRequest);
       xmlHTTPrequest.onreadystatechange = this._onReadyStateChanged;
       xmlHTTPrequest.open('get', url, true);  
       xmlHTTPrequest.send(null);
    }, 

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    }
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesAnonymousInstallStatsService]);
} catch(e) {};
