// Copyright (c) 2009 Yahoo! Inc. 
// All Rights Reserved
// 
// this service provides anonymous usgae statistics for FoxyTunes
// which helps us understand how many people are using FoxyTunes
// the data is only used for stats in an aggregated form
// no personal information is ever disclosed 
// 
// Usage data is only saved for users who have chosen to opt-in to help 
// us improve FoxyTunes
//
// please refer to our Privacy Policy for more details
function FoxyTunesAnonymousUsageStatsService() {
    this._prefService = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);

    try {
       Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
       getService(Components.interfaces.mozIJSSubScriptLoader).
       loadSubScript('chrome://foxytunes/content/utils.js');
    } catch (e) {}

    this._baseURL = 'http://a.analytics.yahoo.com/p.pl';
    this._projectID = '1000967717688';

    this._bUserOptIn = gFoxytunesPreferenceManager.getBoolPref('usage.opt-in.perticipate', false);
    this._bOptInWindowShowed = gFoxytunesPreferenceManager.getBoolPref('usage.opt-in.window-showed', false);

    this._installTime = foxytunesReadSinglePreference('install_time', new Date().getTime());
    this._showOptInTimeoutHours = 24;

    this._userGUID = foxytunesReadSinglePreference('usage.guid', null);
    FTJSLib.include(FTJSLib.jslib_uuid);

    this.wrappedJSObject = this;
}

FoxyTunesAnonymousUsageStatsService.prototype = {
    classID : Components.ID("{D1D7021E-D48B-11DE-A140-312A56D89593}"),

    onUsgaeOpt: function(bOptIn) {
        if (bOptIn != this._bUserOptIn) {
            this._bUserOptIn = bOptIn;
            gFoxytunesPreferenceManager.setBoolPref('usage.opt-in.perticipate', this._bUserOptIn);
        }
        var optStr = bOptIn ? "in" : "out";
        var b = "Opt " + optStr;
        var f = "http://www.foxytunes.com/stats/opt/" + optStr;
        var params = "b=" + encodeURIComponent(b) + "&f=" + encodeURIComponent(f) + "&c=opt"; 
        this._sendAnonymousStats(params);
    },

    onSongPlay:function(trackData) {
        if (!this._bUserOptIn) return;

        if (!trackData || !trackData.title) return;
        var b = trackData.originalTitle;
        var f = "http://www.foxytunes.com/stats/songplay/" + (trackData.artist ? trackData.artist : "-") + "/" + (trackData.album ? trackData.album : "-") + "/" + (trackData.title ? trackData.title : "-");
        var params = "b=" + encodeURIComponent(b) + "&f=" + encodeURIComponent(f) + "&c=songplay";
        this._sendAnonymousStats(params);

    },

    onTwittyTunes: function(consumer, mode) {
        if (!this._bUserOptIn) return;

        var b = "TwittyTunes";
        var f = "http://www.foxytunes.com/stats/twittytunes";
        var params = "b=" + encodeURIComponent(b) + "&f=" + encodeURIComponent(f) + "&c=twittytunes&cf7=" + encodeURIComponent(consumer) + "&cf8=" + mode; 
        this._sendAnonymousStats(params);
    },

    showOptInIfNeeded: function(bForceShow) {
        if (typeof bForceShow == 'undefined') bForceShow = false;

        if (!bForceShow) {
            if (this._bOptInWindowShowed) return;

            var currentTime = new Date();
            var hoursSinceInstall = Math.round((currentTime - this._installTime) / 3600000);
            if (hoursSinceInstall < this._showOptInTimeoutHours) return;
        }

        var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);
        var win = windowMediator.getMostRecentWindow(null);

        // sanity
        if (!win.open) return;

        gFoxytunesPreferenceManager.setBoolPref('usage.opt-in.window-showed', true);
        this._bOptInWindowShowed = true;

        var features = "chrome,titlebar,centerscreen,dialog=no,modal";
        win.open('chrome://foxytunes/content/anonymous-stats/optInWindow.xul', '_blank', features);
        
    },

    // Used in order to prevent the garbage collection
    _onReadyStateChanged: function (){},

    _sendAnonymousStats: function(customParams) {
       var app = Components.classes["@mozilla.org/xre/app-info;1"]
                   .getService(Components.interfaces.nsIXULAppInfo)
                   .QueryInterface(Components.interfaces.nsIXULRuntime);

       ftversion = foxytunesReadSinglePreference('version', '-');

       var locale = '-';
       try {
           locale = this._prefService.getBranch('general.').getCharPref('useragent.locale');
       } catch (e) {};

       if (!this._userGUID) {
           var uuid = new FTJSLib.UUID;
           this._userGUID = uuid.uuidgen();
           foxytunesWriteSinglePreference('usage.guid', this._userGUID);
       }

        var appName = foxytunesGetAppName();

        var url = this._baseURL + '?a=' + this._projectID;
        url += '&cf3=' +  ftversion;
        url += '&cf1=' + appName;
        url += '&cf2=' + app.version;
        url += '&cf4=' + locale;
        url += '&sid=' + this._userGUID;
        if (customParams) {
            url += '&' + customParams;    
        }

       var xmlHTTPrequest = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].
                                    createInstance(Components.interfaces.nsIXMLHttpRequest);
       xmlHTTPrequest.onreadystatechange = this._onReadyStateChanged;
       xmlHTTPrequest.open('get', url, true);  
       xmlHTTPrequest.send(null);
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.nsISupports)) {
               return this;
    }

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
}
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesAnonymousUsageStatsService]);
} catch(e) {};
