/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function MusicUpdatesConsumerLastFm() {
    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
    }

    this._handShakeDetails = {};
    this._propertiesObj = {};
    this._IFTMusicConsumer = Components.interfaces.IFoxyTunesMusicConsumer;
    this._apiKey = 'da514ce15efdbb642008c8694a703a10';
    this._apiBaseURL = 'http://ws.audioscrobbler.com/2.0/?'

    this.homepageRegExp =  new RegExp('^http:\\/\\/[\\w\\.]*last\\.fm\\/');

    this.SetProperty('readableName', 'Last.fm');
    this._username = null;
    this._password = null;
}

MusicUpdatesConsumerLastFm.prototype = {
    classID : Components.ID("{A67C18CC-FC7B-11DD-AFCA-36E955D89593}"),

    init: function() {},
    uninit: function () {},

    OnPlaybackUpdate: function(eventType, trackData, plainText, formattedText, link) {
        var trackDataArr = trackData.split('\t');
        var trackDataObj = {};
        for (var i=0; i<trackDataArr.length-1; i+=2) {
            trackDataObj[trackDataArr[i]] = trackDataArr[i+1];
        }
        
        switch (eventType) {
            case this._IFTMusicConsumer.EVENT_TRACK_PLAYING: 
                this._nowPlaying(trackDataObj);
                break;
            case this._IFTMusicConsumer.EVENT_TRACK_FINISHED_PLAYING: 
                this._submitTrack(trackDataObj);
                break;
        }
    },

    OnPlaybackStopped: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    SetUserAuth: function(username, password) {
        this._username = username;
        this._password = password;
        this.startHandshake(username, foxytunesMd5(password));
    },

    SendMusicUpdate: function(trackData, plainText, formattedText, link) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    SendGenericUpdate: function(topic, arguments) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    GetAuthWindowURL:function() {
        return 'chrome://foxytunes/content/music-updates/auth-windows/basicAuth.xul';
    },

    IsInstalled: function() {
        return true;
    },

    IsRunning: function() {
        var win = foxytunesGetMostRecentBrowserWindow();
        if (!win) return false; 

        try {
            var tabBrowser = win.getBrowser();
            var doc = tabBrowser.contentDocument;

            if (doc.location.href.match(this.homepageRegExp)) {
                return true;
            }
        } catch(e) {}
        
        return false;
    },

    RunService: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    }, 

    GetCapabilities: function() {
        return (this._IFTMusicConsumer.CONSUMER_SUPPORT_AUTO_UPDATES | 
                this._IFTMusicConsumer.CONSUMER_REQUIRES_AUTH);
    },

    SetProperty: function(propertyName, propertyValue) {
        this._propertiesObj[propertyName] = propertyValue;
    },

    GetProperty: function(propertyName) {
        return this._propertiesObj[propertyName];
    },

    SetProperties: function(propertyList) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    GetProperties: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    // Private
    _nowPlaying: function(trackData) {
        if (!this._handShakeDetails || !this._handShakeDetails['session']) return;

        var params = 's='  + this._handShakeDetails['session'] + 
                     '&a=' + (trackData.artist ? encodeURIComponent(trackData.artist) : '') +
                     '&t=' + (trackData.title ? encodeURIComponent(trackData.title) : '') +
                     '&b=' + (trackData.album ? encodeURIComponent(trackData.album) : '') +
                     '&l=' + (trackData.length ? trackData.length : '') +
                     '&n=&m=';
        var url =  this._handShakeDetails['np-url'];
        this.sendToLastFm({'method': 'POST', 'url': url, 'params': params});
    },

    _submitTrack: function(trackData) {
        if (!this._handShakeDetails || !this._handShakeDetails['session']) return;

        var me = this;
        var submitTrack = function(trackLength) {
        var time = new Date();
            time = Math.round((time.getTime() + (time.getTimezoneOffset() * 60)) / 1000);
            var params = 's=' + me._handShakeDetails['session'] + 
            '&a[0]=' + (trackData.artist ? encodeURIComponent(trackData.artist) : '') +
            '&t[0]=' + (trackData.title ? encodeURIComponent(trackData.title) : '') +
            '&b[0]=' + (trackData.album ? encodeURIComponent(trackData.album) : '') +
            '&i[0]=' + time + 
            '&o[0]=P' +
              '&l[0]=' + trackLength +
            '&r[0]=&m[0]=&n[0]=';
            var url =  me._handShakeDetails['submission-url']; 
            me.sendToLastFm({'method': 'POST', 'url': url, 'params': params});
        }

        if (trackData.length) {
            submitTrack(trackData.length);
        } else {
            if (!trackData.artist || !trackData.title) return;

            var trackInfoCB = function(xmlHTTPrequest) {
                if (xmlHTTPrequest.status != 200) return;

                var xml = xmlHTTPrequest.responseXML;
                if (!xml) return;

                var length = xml.getElementsByTagName('duration')[0];
                if (!length) return;

                length = length.firstChild.nodeValue;
                submitTrack(length/1000);
            }

            var trackInfoURL = this._apiBaseURL + 'method=track.getinfo&api_key=' + this._apiKey + 
                                '&artist=' + encodeURIComponent(trackData.artist) + '&track=' + encodeURIComponent(trackData.title);

            foxytunesHTTPRequest({'method': 'GET', 'url': trackInfoURL}, trackInfoCB);
        }
    },

    sendToLastFm: function(request) {
         var me = this;
         var lastFmCB = function(xmlHTTPrequest) {
             if (xmlHTTPrequest.responseText.indexOf("BADSESSION") > -1) {
                 if (!me._username || !me._password) return;
                 me.startHandshake(me._username, foxytunesMd5(me._password), function() { foxytunesHTTPRequest(request); });
             } 
         }

        foxytunesHTTPRequest(request, lastFmCB);
    },

    startHandshake: function(username, password, callback) {
        var time = new Date();
        time = Math.round((time.getTime() + (time.getTimezoneOffset() * 60)) / 1000);
        var url = 'http://post.audioscrobbler.com' + 
	                    '?hs=true' +
          	            '&p=1.2.1' +
                        '&c=fxy&v=1.0' + 
                        '&u=' + username +
                        '&t=' + time +
                        '&a=' + foxytunesMd5(password + time.toString());

        var me = this;
        foxytunesHTTPRequest({'url': url}, function(xmlHTTPrequest) { me.handShakeDone(xmlHTTPrequest, callback); });
    },

    handShakeDone : function(xmlHTTPrequest, callback) {
        this._handShakeDetails = {};
        this.parseHandShakeReply(xmlHTTPrequest.responseText);

        if (this._handShakeDetails['session'] == 'FAILED') {
            return;
        }

        if (callback) callback();
    },

    parseHandShakeReply : function(data) {
        var lines = data.split('\n');
        this._handShakeDetails['status'] = lines[0];
        if (this._handShakeDetails['status'] == 'OK') {
            this._handShakeDetails['session'] = lines[1];
            this._handShakeDetails['np-url'] = lines[2];
            this._handShakeDetails['submission-url'] = lines[3];
        }
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.IFoxyTunesMusicConsumer) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
    }

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
}
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([MusicUpdatesConsumerLastFm]);
} catch(e) {};
