/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function MusicUpdatesConsumerTwitter() {
    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/oauth/oauthUtils.js");
    }

    this._propertiesObj = {};
    this._IFTMusicConsumer = Components.interfaces.IFoxyTunesMusicConsumer;

    this._httpRequestTimer = Components.classes["@mozilla.org/timer;1"].
				                 		createInstance(Components.interfaces.nsITimer); 

    this._apiKey = 'WTe0nikR4Iu0whKBWg';
    this._baseURL = 'http://api.twitter.com/1/statuses/';

    this._oauthUtils = new OAuthUtils(this._apiKey, 'twitter');
    this._oauthUtils.setBaseURL('https://api.twitter.com/oauth/');
    this._oauthUtils.setUrlEndPoints({'access_token': 'access_token'});

    this._password = null;
    this._username = null;

    this.SetProperty('maxCharacters', 140);

    this.homepageRegExp =  new RegExp('^http:\\/\\/[\\w\\.]*twitter\\.com\\/');

    this.wrappedJSObject = this;
}

MusicUpdatesConsumerTwitter.prototype = {
    classID : Components.ID("{E88F4780-14AD-11DE-AD18-408955D89593}"),
    
    init: function() {}, 
    uninit: function () {}, 

    OnPlaybackUpdate: function(eventType, trackData, plainText, formattedText, link) {
       throw Components.results.NS_ERROR_NOT_IMPLEMENTED; 
    },

    OnPlaybackStopped: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED; 
    },

    SetUserAuth: function(username, authDetails) {
        this._username = username;
        var tmpArr = authDetails.split('\t');
        var authDetails = {
            "accessToken": tmpArr[0],
            "accessTokenSecret": tmpArr[1]
        }
        this._oauthUtils.setAuthDetails(authDetails);
    },

    SendMusicUpdate: function(trackData, plainText, formattedText, link) {
        var me = this;
        
        var tinyURLCB = function(shortLink) {
            // Case that tinyURL returned an error
            // just use the full link
            var linkToSend = shortLink ? shortLink :  link;

            me.postToTwitter(plainText + " " + linkToSend);
        };

        var sendHttpRequestOnTimer = {
            notify: function (aTimer) {
                if (aTimer != me._httpRequestTimer) return;
                if (link) {
                    foxytunesGetTinyURL(link, tinyURLCB);
                } else {
                    me.postToTwitter(plainText);
                }
            }
        };

        // Calling httprequest with timer because the call is initiated from a window
        // and when the window closes it creates a problem to httprequest 
        this._httpRequestTimer.initWithCallback(sendHttpRequestOnTimer, 1000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);  
    },

    SendGenericUpdate: function(topic, arguments) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    GetAuthWindowURL:function() {
        return 'chrome://foxytunes/content/music-updates/auth-windows/twitterAuth.xul';
    },

    IsInstalled: function() {
        return true;
    },

    IsRunning: function() {
        var win = foxytunesGetMostRecentBrowserWindow();
        if (!win) return false; 

        try {
            var tabBrowser = win.getBrowser();
            var doc = tabBrowser.contentDocument;

            if (doc.location.href.match(this.homepageRegExp)) {
                return true;
            }
        } catch(e) {}
        
        return false;
    },

    RunService: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    }, 

    GetCapabilities: function() {
        return (this._IFTMusicConsumer.CONSUMER_SUPPORT_MANUAL_UPDATES | 
                this._IFTMusicConsumer.CONSUMER_REQUIRES_AUTH);
    },

    SetProperty: function(propertyName, propertyValue) {
        this._propertiesObj[propertyName] = propertyValue;
    },

    GetProperty: function(propertyName) {
        return this._propertiesObj[propertyName];
    },

    SetProperties: function(propertyList) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    GetProperties: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    // Private
    postToTwitter : function(text, callback) {
        var params = {'status': text};
        var url = this._baseURL + 'update.xml';
        var signedURL = this._oauthUtils.getAPIURL("POST", url, params); 
        return foxytunesHTTPRequest({'method': 'POST', 'url': signedURL}, callback);
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.IFoxyTunesMusicConsumer) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
    }

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
}
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([MusicUpdatesConsumerTwitter]);
} catch (e) {};
