/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function MusicUpdatesConsumesFacebook() {
    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/json_parse_state.js");
    }

    // Calling httprequest with timer because the call is initiated from a window
    // and when the window closes it creates a problem to httprequest 
    this._httpRequestTimer = Components.classes["@mozilla.org/timer;1"].
				                 		createInstance(Components.interfaces.nsITimer); 

    this._propertiesObj = {};
    this._IFTMusicConsumer = Components.interfaces.IFoxyTunesMusicConsumer;

    this._unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                getService(Components.interfaces.nsIScriptableUnicodeConverter);
    this._unicodeConverter.charset = 'UTF-8';
    this._signingService =  Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesSigningService;1"]
                                     .getService(Components.interfaces.IFoxyTunesSigningService);

    this._apiKey = '60770b5cf0420e5f9ef9b21788a8e361';
    this._token = null;
    this._sessionKey = null;
    this._sessionSecret = null;

    this._apiURL = 'http://api.facebook.com/restserver.php';
    this._loginURLBase = 'http://www.facebook.com/login.php?popup=1&api_key=' + this._apiKey + '&v=1.0&auth_token=';
    this.extPermURLBase = 'http://www.facebook.com/authorize.php?popup=1&api_key=' + this._apiKey + '&v=1.0&ext_perm=';

    this.homepageRegExp =  new RegExp('^http:\\/\\/[\\w\\.]*facebook\\.com\\/');

    this.wrappedJSObject = this;
}

MusicUpdatesConsumesFacebook.prototype = {
    classID : Components.ID("{F1E2BC78-3027-11DE-87F4-BC5455D89593}"),
    
    init: function() {}, 
    uninit: function () {}, 

    OnPlaybackUpdate: function(eventType, trackData, plainText, formattedText, link) {
       throw Components.results.NS_ERROR_NOT_IMPLEMENTED; 
    },

    OnPlaybackStopped: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED; 
    },

    SetUserAuth: function(userName, sessionKeyAndSecret) {
        var tmpArr = sessionKeyAndSecret.split('\t');
        this._sessionKey = tmpArr[0];
        this._sessionSecret = tmpArr[1];
    },

    SendMusicUpdate: function(trackData, plainText, formattedText, link) {
      var me = this;
      var tinyURLCB = function(shortLink) {
          // In case of error in tinyURL
          if (!shortLink) return;

          setFBStatus(plainText, shortLink);
      };

      function setFBStatus(text, link) {
          var method = 'facebook.users.setStatus';
          var textWithLink = text + (link ? (" " + link) : "");
          var params = {
            status: textWithLink,
            session_key: me._sessionKey,
            call_id: new Date().getTime(),
          };
          me.callFacebookAPI(method, params);
      };

      var sendHttpRequestOnTimer = {
        notify: function (aTimer) {
            if (aTimer != me._httpRequestTimer) return;
            if (link) {
                foxytunesGetTinyURL(link, tinyURLCB);
            } else {
                setFBStatus(plainText, null);
            }
        }
      };

      // Calling httprequest with timer because the call is initiated from a window
      // and when the window closes it creates a problem to httprequest 
      this._httpRequestTimer.initWithCallback(sendHttpRequestOnTimer, 1000, Components.interfaces.nsITimer.TYPE_ONE_SHOT);  
    },

    SendGenericUpdate: function(topic, arguments) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    GetAuthWindowURL:function() {
        return 'chrome://foxytunes/content/music-updates/auth-windows/facebookAuth.xul';
    },

    IsInstalled: function() {
        return true;
    },

    IsRunning: function() {
        var win = foxytunesGetMostRecentBrowserWindow();
        if (!win) return false; 

        try {
            var tabBrowser = win.getBrowser();
            var doc = tabBrowser.contentDocument;

            if (doc.location.href.match(this.homepageRegExp)) {
                return true;
            }
        } catch(e) {}
        
        return false;
    },

    RunService: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    }, 

    GetCapabilities: function() {
        return (this._IFTMusicConsumer.CONSUMER_SUPPORT_MANUAL_UPDATES | 
                this._IFTMusicConsumer.CONSUMER_REQUIRES_AUTH);
    },

    SetProperty: function(propertyName, propertyValue) {
        this._propertiesObj[propertyName] = propertyValue;
    },

    GetProperty: function(propertyName) {
        return this._propertiesObj[propertyName];
    },

    SetProperties: function(propertyList) {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    GetProperties: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    // Private
    createToken: function(callback) {
        var method = 'facebook.auth.createToken'; 
        this.callFacebookAPI(method, null, callback);
    },

    getSession: function(token, callback) {
        var method = 'facebook.auth.getSession';
        var params = {
            auth_token: token,
        }
        this.callFacebookAPI(method, params, callback);
    },

    getUserName: function(callback) {
        var me = this;
        var loggedInUserCB = function(userID) {
            var method = 'facebook.users.getInfo';
            var params = {
                session_key: me._sessionKey,
                uids: userID,
                fields: 'name',
            }
            me.callFacebookAPI(method, params, callback);
        }            
        this.getLoggedInUser(loggedInUserCB);
    },

    getLoggedInUser: function(callback) {
        var method = 'facebook.users.getLoggedInUser';
        var params = {
            session_key: this._sessionKey
        }
        this.callFacebookAPI(method, params, callback);
    },

    getExtPermURL: function(perm) {
        return this.extPermURLBase + perm;
    },

    hasExtPerm: function(perm, callback) {
        var method = 'facebook.users.hasAppPermission';
        var params = {
            session_key: this._sessionKey,
            ext_perm: perm
        };
        this.callFacebookAPI(method, params, callback);
    },

    sign: function(method, params) {
        var keys = new Array();
        for (var i in params) keys.push(i);
        keys.sort();

        var req_str = '';
        for (i=0; i<keys.length; i++) req_str += keys[i] + '=' + params[keys[i]];

        var secret = ((method == "facebook.auth.createToken") || (method == "facebook.auth.getSession")) ? null : this._sessionSecret;
        params.sig = this._signingService.SignString("facebook", this._unicodeConverter.ConvertFromUnicode(req_str), secret);

        return params;
    },

    getParamsStr: function(method, params) {
        var params = this.sign(method, params);
        var c = 0;
        var ret = '';
        for (var i in params) {
            if (c++!=0) ret += '&';
            ret += encodeURIComponent(i) + '=' + encodeURIComponent(params[i]);
        }
        return ret;
    },

    callFacebookAPI: function(method, params, callback) {
        if (!params) var params = {};
        params.method = method;
        params.api_key = this._apiKey;
        params.format = 'JSON';
        params.v = '1.0';
        var paramsStr = this.getParamsStr(method, params);
        var url = this._apiURL + '?' + paramsStr;

        var fbCallback = function(xmlHTTPrequest) {
            if (!callback || !xmlHTTPrequest) return;

            var contentsArr;
            if (xmlHTTPrequest.status != 200) {
                // In case of error
                contentsArr = {};
                contentsArr.error_code = xmlHTTPrequest.status;
                contentsArr.error_msg = xmlHTTPrequest.responseText;
            } else {
                contentsArr = json_parse('[' + xmlHTTPrequest.responseText + ']');
            }
            callback(contentsArr);
        }
        foxytunesHTTPRequest({'url': url}, fbCallback);
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.IFoxyTunesMusicConsumer) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
    }

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
}
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([MusicUpdatesConsumesFacebook]);
} catch (e) {};
