/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

function FoxytunesMusicUpdatesService() {
    this.observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);
    this.observerService.addObserver(this, "foxytunes-internal-track-data", false);

    this._foxytunesService = Components.classes['@foxytunes.org/FoxyTunesEngine/FoxyTunesService;1']
                                .getService(Components.interfaces.IFoxyTunesService);
    this._foxyTunesAnonymousUsageStatsService = Components.classes["@foxytunes.org/FoxyTunesEngine/FoxyTunesAnonymousUsageStatsService;1"]
                .getService().wrappedJSObject;

    this.prefService = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);
    this.prefBranch  = this.prefService.getBranch("foxytunes.");
    this._prefBase = 'music_updates.';

    this._newTrackPlayingTimer = Components.classes["@mozilla.org/timer;1"].
				            		createInstance(Components.interfaces.nsITimer); 
    this._newTrackPlayingDelay = 10000;

    this._trackConsideredPlayedTimer = Components.classes["@mozilla.org/timer;1"].
				                 		createInstance(Components.interfaces.nsITimer); 
    this._trackConsideredPlayedDefaultDelay = 120000;
    this._bLastTrackConsideredPlayed = false;

    this._bSendUnknownMediaTypes = false;
    this.getSendingUnknownMediaTypes();

    // Password Manager does not exist in Firefox 3
    var CC_passwordManager = Components.classes["@mozilla.org/passwordmanager;1"];
    var CC_loginManager = Components.classes["@mozilla.org/login-manager;1"];
                    
    this._bUsingLoginManager = CC_passwordManager ? false : true;
    this._loginManagerHostSuffix = '.twittytunes.foxytunes.com';

    if (this._bUsingLoginManager) {
        this.loginManager = CC_loginManager.getService(Components.interfaces.nsILoginManager);
    } else {
        this.loginManager = CC_passwordManager.getService(Components.interfaces.nsIPasswordManager);
    }

    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
    }

    this._autoUpdatesTemplate = foxytunesGetLocaleStringExternalfunction('nowPlaying', [ '%artist-track%' ]);

    this._currentTrackData = null;

    this._consumersDetails = new Object();

    this._IFTMusicConsumer = Components.interfaces.IFoxyTunesMusicConsumer;

    this._trackDataStringSeperator = '\t';

    this._numOfActivatedConsumers = {};
    this._numOfActivatedConsumers["manual"] = 0;
    this._numOfActivatedConsumers["auto"] = 0;

    // Music updates hints
    this._installTime = foxytunesReadSinglePreference('install_time', new Date().getTime());
    this._hintsShown = {};
    this._hintsText = {};
    this._hintsText['last.fm'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsLastFm');
    this._hintsText['twitter'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsTwitter');
    this._hintsText['facebook'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsFacebook');
    this._hintsText['yahooupdates'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsYahooUpdates');
    this._hintsText['yahoomessenger'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsYahooMessenger');
    this._hintsText['livemessenger'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsLiveMessenger');
    this._hintsText['skype'] = foxytunesGetLocaleStringExternalfunction('twittytunesHintsLiveSkype');

    this.wrappedJSObject = this;

    this.init();
}

FoxytunesMusicUpdatesService.prototype = {
    classID : Components.ID("{5679A7CA-FC5A-11DD-AD48-055956D89593}"),

    // IFoxyTunesMusicConsumer events
    onPlaybackStopped: function() {
        for (consumer in this._consumersDetails) {
            if (!this._consumersDetails[consumer].bIsUpdatesActivated["auto"]) continue;
            try {
                this._consumersDetails[consumer].service.OnPlaybackStopped();
            } catch (e) {};
        }
    },

    onPlaybackUpdate: function(eventType, trackData) {
        // Reporting last track finished playing when getting a new track
        // and the last one has played more than it's half time
        if (eventType == this._IFTMusicConsumer.EVENT_TRACK_CONSIDERED_PLAYED) {
            this._bLastTrackConsideredPlayed = true;
        }
        var trackLink = foxytunesGetTrackDataLink(trackData);
        var musicUpdateText = this.getMusicUpdateText(trackData, trackLink, 'html', this._autoUpdatesTemplate);
        var trackDataStr = this.trackDataToString(trackData);
        for (consumer in this._consumersDetails) {
            if (!this._consumersDetails[consumer].bIsUpdatesActivated["auto"]) continue;
            try {
                this._consumersDetails[consumer].service.OnPlaybackUpdate(eventType, trackDataStr, musicUpdateText[0], musicUpdateText[1], trackLink);
            } catch (e) { dump("musicUpdatesService: onPlaybackUpdate error - " + e + "\n");}
            if (eventType == this._IFTMusicConsumer.EVENT_TRACK_PLAYING) {
                this._foxyTunesAnonymousUsageStatsService.onTwittyTunes(consumer, "auto");
            }
        }

        if (eventType == this._IFTMusicConsumer.EVENT_TRACK_PLAYING) {
            this.showMusicUpdatesHintIfNeeded();
            this._foxyTunesAnonymousUsageStatsService.onSongPlay(trackData);
        }
    },

    sendMusicUpdate:function (trackData, plainText, formattedText, link) {
        for (consumer in this._consumersDetails) {
            if (!this._consumersDetails[consumer].bIsUpdatesActivated["manual"]) continue;

            try {
                var trackDataStr = this.trackDataToString(trackData);
                this._consumersDetails[consumer].service.SendMusicUpdate(trackDataStr, plainText, formattedText, link);
            } catch (e) { dump("musicUpdatesService: sendMusicUpdate error - " + e + "\n");}
			this._foxyTunesAnonymousUsageStatsService.onTwittyTunes(consumer, "manual");
        }
    },

    // Consumer Configuration
    getConsumerPrefBase: function(consumer) {
        return this._prefBase + 'consumers.' + consumer + '.';
    },

    isConsumerNeedAuth: function(consumer) {
        return ((this._consumersDetails[consumer].capabilities & this._IFTMusicConsumer.CONSUMER_REQUIRES_AUTH) != 0);
    },

    isConsumerSupportManualUpdates: function(consumer) {
        return ((this._consumersDetails[consumer].capabilities & this._IFTMusicConsumer.CONSUMER_SUPPORT_MANUAL_UPDATES) != 0);
    },

    isConsumerSupportAutoUpdates: function(consumer) {
        return ((this._consumersDetails[consumer].capabilities & this._IFTMusicConsumer.CONSUMER_SUPPORT_AUTO_UPDATES) != 0);
    },

    isConsumerHasAuthDetails: function(consumer) {
        var isConsumerHasAuthDetails = false;
        try {
            var authDetails = this.getConsumerAuthDetails(consumer);
            if (authDetails && authDetails[0] && authDetails[1]) {
                isConsumerHasAuthDetails = true;
            }
        } catch(e) {};
        return isConsumerHasAuthDetails;
    },

    getConsumerFromContractID: function(contract) {
        try {
	        var contractArray = contract.split("/");
	        if (contractArray[1].toLowerCase() != 'musicupdatesconsumer') {
		        return null;
	        }
	        var subModule = contractArray[2];
	        var subModuleArray = subModule.split(";");
	        return subModuleArray[0];
        } catch (e) {
	        return null;
        }
    },

    getConsumerHost: function(consumer) {
        return 'http://' + consumer + this._loginManagerHostSuffix;
    },

    isUpdatesActivated: function(consumer, mode) {
        var bUpdatesActivated = false;
        try {
            bUpdatesActivated = this.prefBranch.getBoolPref(this.getConsumerPrefBase(consumer) + 'send_' + mode + '_updates');
        } catch (e) {};
        
        return bUpdatesActivated;
    },

    getNumberOfActivatedConsumers: function(mode) {
        return this._numOfActivatedConsumers[mode];
    },

    setConsumerAuthDetails: function(consumer, username, password) {
        var hostName = this.getConsumerHost(consumer);
        if (this._bUsingLoginManager) {
            var nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
                                             Components.interfaces.nsILoginInfo, "init");
            var loginInfo = new nsLoginInfo(hostName, hostName, null, username, password, "", "");
            this.loginManager.addLogin(loginInfo);
        } else {
            this.loginManager.addUser(hostName, username, password);
        }
    },

    getConsumerAuthDetails: function(consumer) {
        var hostName = this.getConsumerHost(consumer); 
        if (this._bUsingLoginManager) {
            var logins = this.loginManager.findLogins({}, hostName, hostName, null);
            if (logins.length == 0) return null;
            return [logins[0].username, logins[0].password];
        }

         // old passwordManager code (FF < 3.0)
        var e = this.loginManager.enumerator;
        while (e.hasMoreElements()) {
            try {
                var pass = e.getNext().QueryInterface(Components.interfaces.nsIPassword);
                if (pass.host == hostName) {
                    return [pass.user, pass.password];
                }
            } catch (ex) {}
        }

        return null;
    },

    removeConsumerAuthDetails: function(consumer) {
       var hostName = this.getConsumerHost(consumer); 
       if (this._bUsingLoginManager) {
            var logins = this.loginManager.findLogins({}, hostName, hostName, null);
            if (logins && logins[0]) {
                this.loginManager.removeLogin(logins[0]);
            } 
        } else {
            this.loginManager.removeReject(hostName); 
        }
    },

    sendAuthDetailsToConsumer: function(consumer) {
        var authDetails = this.getConsumerAuthDetails(consumer);
        if (!authDetails) return;

        try {
        this._consumersDetails[consumer].service.SetUserAuth(authDetails[0], authDetails[1]);
        } catch (e) {
            dump("musicUpdatesService: Failed to SetUserAuth for " + consumer + ", " + e + "\n");
        }
    },

    // Consumers Info
    getConsumersDetails: function() {
        return this._consumersDetails;
    },

    setConumerUpdatesActiveState: function(consumer, mode, bActivate) {
        if (this._consumersDetails[consumer].bIsUpdatesActivated[mode] == bActivate) return;

        if (bActivate) {
            this.initConsumer(consumer);
        } else {
            this.uninitConsumer(consumer);
        }
        this._consumersDetails[consumer].bIsUpdatesActivated[mode] = bActivate;
        this._numOfActivatedConsumers[mode] += bActivate ? 1 : -1;
        this.prefBranch.setBoolPref(this.getConsumerPrefBase(consumer) + 'send_' + mode + '_updates', bActivate);

        if (mode == "auto") {
            this.observerService.notifyObservers(null, "foxytunes-internal-twittytunes", "auto-mode-consumer-activation-toggle");
        }
    },

    getCurrentMaxAllowedCharacters: function() {
        var max = Number.MAX_VALUE;
        for (consumer in this._consumersDetails) {
            if (!this._consumersDetails[consumer].bIsUpdatesActivated["manual"]) continue;

            try {
                var currMax = this._consumersDetails[consumer].service.GetProperty('maxCharacters');
                if (parseInt(currMax) && (currMax < max)) max = currMax;
            } catch (e) {}
        }
        return (max == Number.MAX_VALUE) ? -1 : max;
    },

    // Service configuration
    setSendingUnknownMediaTypes: function(bAllow) {
        this._bSendUnknownMediaTypes = bAllow;
        this.prefBranch.setBoolPref(this._prefBase + 'auto_send_unknown_media_types', bAllow);
    },

    getSendingUnknownMediaTypes: function() {
        var bAllow = false;
        try {
            bAllow = this.prefBranch.getBoolPref(this._prefBase + 'auto_send_unknown_media_types');
        } catch(e) {};
        this._bSendUnknownMediaTypes = bAllow;
        return bAllow;
    },

    // Private
    init: function() {
        for (var classContract in Components.classes) {
            var consumerName = this.getConsumerFromContractID(classContract); 
            if (!consumerName) continue;

            var consumer = consumerName.toLowerCase();
            this._consumersDetails[consumer] = new Object();
            this._consumersDetails[consumer].classContract = classContract;
            this._consumersDetails[consumer].service = Components.classes[classContract].getService(Components.interfaces.IFoxyTunesMusicConsumer);
            this._consumersDetails[consumer].capabilities = 0;
            try {
                this._consumersDetails[consumer].capabilities = this._consumersDetails[consumer].service.GetCapabilities();
            } catch (e) {};
            this._consumersDetails[consumer].bIsUpdatesActivated = {};
            var modes = ["auto", "manual"];
            for (var i in modes) {
                this._consumersDetails[consumer].bIsUpdatesActivated[modes[i]] = this.isUpdatesActivated(consumer, modes[i]);
                if (this._consumersDetails[consumer].bIsUpdatesActivated[modes[i]]) {
                    // If consumer need auth but for some reason auth details are missing - turn activation off
                    if (this.isConsumerNeedAuth(consumer) && !this.isConsumerHasAuthDetails(consumer)) {
                        this._consumersDetails[consumer].bIsUpdatesActivated[modes[i]] = false;
                        continue;
                    }
                    this._numOfActivatedConsumers[modes[i]]++;
                }
            }
            this._consumersDetails[consumer].needAuth = this.isConsumerNeedAuth(consumer);
            if (this._consumersDetails[consumer].needAuth) {
                this._consumersDetails[consumer].authWindowURL = null;
                try {
                this._consumersDetails[consumer].authWindowURL = this._consumersDetails[consumer].service.GetAuthWindowURL();
                } catch (e) {};
            }
            this._consumersDetails[consumer].bIsInited = false;
            var readableName;
            try {
              readableName = this._consumersDetails[consumer].service.GetProperty('readableName');
            } catch (e) {};
            this._consumersDetails[consumer].readableName = readableName ? readableName : consumerName;
            try {
                this._consumersDetails[consumer].readableNameManual =  this._consumersDetails[consumer].service.GetProperty('readableNameManual');
                this._consumersDetails[consumer].readableNameAuto =  this._consumersDetails[consumer].service.GetProperty('readableNameAuto');
            } catch(e) {}; 
            if (this._consumersDetails[consumer].bIsUpdatesActivated["auto"] || 
                this._consumersDetails[consumer].bIsUpdatesActivated["manual"]) {
                this.initConsumer(consumer);
            }
        }

        // Convert old Twitter users from basic to xAuth
        this.convertTwitterFromBasicToXAuthIfNeeded();
    },

    initConsumer: function(consumer) {
         if (!this._consumersDetails[consumer].bIsInited) {
            try {
                this._consumersDetails[consumer].service.init();
                this._consumersDetails[consumer].bIsInited = true;
            } catch (e) {
                dump("musicUpdatesService: Failed to init " + consumer + ", " + e + "\n");
                return;
            }
        }
        if (this.isConsumerNeedAuth(consumer)) {
                this.sendAuthDetailsToConsumer(consumer);
            }
    },

    uninitConsumer: function(consumer) {
        if (!this._consumersDetails[consumer].bIsInited) return;

        try {
            this._consumersDetails[consumer].service.uninit();
            this._consumersDetails[consumer].bIsInited = false;
        } catch (e) {
            dump("musicUpdatesService: Failed to uninit " + consumer + ", " + e + "\n");
        }
    },

    notify: function(aTimer) {
        if (aTimer == this._newTrackPlayingTimer) {
            this.onPlaybackUpdate(this._IFTMusicConsumer.EVENT_TRACK_PLAYING, this._currentTrackData);
        }

        if (aTimer == this._trackConsideredPlayedTimer) {
            this.onPlaybackUpdate(this._IFTMusicConsumer.EVENT_TRACK_CONSIDERED_PLAYED, this._currentTrackData);
        }
    },

    observe: function(subject, topic, data) {
        if ((topic != "foxytunes-internal-track-data") ||
            (data != "track-title-changed-internal")) {
            return;
        }
        var trackData = this.getCurrentTrackData();
        var title = trackData.title;
        

        if (this._currentTrackData) {
           if (this._currentTrackData.title == title) return;

           // Cancel old timers if a new track arrives 
           this._newTrackPlayingTimer.cancel();
           this._trackConsideredPlayedTimer.cancel();

           // If a new track arrives and the old one is considered player > notify that the last track finished playing
           if (this._bLastTrackConsideredPlayed) {
               this.onPlaybackUpdate(this._IFTMusicConsumer.EVENT_TRACK_FINISHED_PLAYING, this._currentTrackData);
           }
           this._bLastTrackConsideredPlayed = false;
        }
        this._currentTrackData = trackData;

        // Ignore video tracks
        if (trackData.isVideo == 'yes') return;
        if (!this._bSendUnknownMediaTypes && trackData.isVideo == 'unknown') return;

        if ((title == null) || (title == "")) {
            this.onPlaybackStopped();
            return;
        }

        var trackConsideredPlayedDelay = this._trackConsideredPlayedDefaultDelay;
        if (trackData.length) {
            if (trackData.length < 30) return;
            trackConsideredPlayedDelay = Math.round(trackData.length / 2) * 1000;
        }

        this._newTrackPlayingTimer.initWithCallback(this, this._newTrackPlayingDelay, Components.interfaces.nsITimer.TYPE_ONE_SHOT);  
        this._trackConsideredPlayedTimer.initWithCallback(this, trackConsideredPlayedDelay, Components.interfaces.nsITimer.TYPE_ONE_SHOT);  

        this.onPlaybackUpdate(this._IFTMusicConsumer.EVENT_NEW_TRACK, this._currentTrackData);
    },

    getMusicUpdateText: function(trackData, trackLink, formatType, template) {
        var trackTitle = gFoxyTunesSanitizer.replaceTriangularBrackets(trackData.originalTitle);
        var musicUpdateText = template.replace(/%artist-track%/g, trackTitle);
        var musicUpdateFormattedText = '';

        switch (formatType) {
            case 'html': 
                if (trackLink) {
                 var artistTrack = '<a href="' + trackLink + '">' + trackTitle + '</a>';
                    musicUpdateFormattedText = template.replace(/%artist-track%/g, artistTrack);
                } else {
                    musicUpdateFormattedText = musicUpdateText;
                }
                 break;
        }
                            
        return [musicUpdateText, musicUpdateFormattedText]; 
    },

    getCurrentTrackData: function() {
        var trackData = this._foxytunesService.GetCurrentTrackData(true);
        return trackData.wrappedJSObject; 
    },

    trackDataToString: function(trackData) {
        var trackDataStr = '';
        var trackDataAttr = new Array('artist', 'album', 'title', 'length');
        for (var i=0; i<trackDataAttr.length; i++) { 
            var attr = trackDataAttr[i];
            if (trackData[attr]) {
                trackDataStr += attr + this._trackDataStringSeperator  + trackData[attr] + this._trackDataStringSeperator;
            }
        }

        return trackDataStr;
    },

    // Music updates hints
    shouldShowMusicUpdatesHints: function() {
        // Auto is on - don't show hints
        if (this.getNumberOfActivatedConsumers("auto") > 0) return false;

        // Don't show again is on - don't show hints
        var bDontShowAgain = (foxytunesReadSinglePreference('xul-tooltip.foxytunes-music-update-hint.dont-show-again', 'false') == 'true');
        if (bDontShowAgain) return false;

        // Less than 7 days since installation - don't show hints
        var currentTime = new Date();
        var daysSinceInstall = Math.round((currentTime - this._installTime) / 86400000);
        if (daysSinceInstall < 7) return false;

        return true;
    },

    shouldShowHintForConsumer: function(consumerName) {
        if (typeof this._hintsText[consumerName] == 'undefined') return false;

        if (typeof this._hintsShown[consumerName] == 'undefined') {
            this._hintsShown[consumerName] = (foxytunesReadSinglePreference('music_updates.hints.' + consumerName + '.displayed', 'false') == 'true');
        }

        return !this._hintsShown[consumerName];
    },

    showMusicUpdateHintForConsumer: function(consumerName) {
       this.observerService.notifyObservers(null, "foxytunes-internal-twittytunes-hint", this._hintsText[consumerName]);
       foxytunesWriteSinglePreference('music_updates.hints.' + consumerName + '.displayed', 'true'); 
       this._hintsShown[consumerName] = true;
    },

    showMusicUpdatesHintIfNeeded: function() {
        if (!this.shouldShowMusicUpdatesHints()) return;

        for (var consumer in this._consumersDetails) {
            var isRunning = false;
            try {
                isRunning = this._consumersDetails[consumer].service.IsRunning();
            } catch (e) {};
            if (isRunning) {
                if (!this.shouldShowHintForConsumer(consumer)) continue;
                this.showMusicUpdateHintForConsumer(consumer);
                break;
            }
        }
    },

    convertTwitterFromBasicToXAuthIfNeeded: function() {
        // Sanity
        if (typeof this._consumersDetails["twitter"] == "undefined") return;

        var bConvertNeeded = false;
        try {
            bConvertNeeded = this.prefBranch.getBoolPref(this.getConsumerPrefBase("twitter") + 'convert_to_xauth_needed');
        } catch (e) {
            bConvertNeeded = true;
        };
        if (!bConvertNeeded) return;
        this.prefBranch.setBoolPref(this.getConsumerPrefBase("twitter") + 'convert_to_xauth_needed', false);

        if (!this.isConsumerHasAuthDetails("twitter")) return; // No basic auth details for Twitter - no need to convert

        var authDetails = this.getConsumerAuthDetails("twitter");
        var username = authDetails[0];
        var password = authDetails[1];

        var me = this;
        var onGotAccessToken = function(resp) {
            // On error just invalidate the auth details
            if (!resp || !resp["oauth_token"] || !resp["oauth_token_secret"]) {
                me.removeConsumerAuthDetails("twitter");
                // If Twitter is currenlt acctivated - turn it off
                me.setConumerUpdatesActiveState("twitter", "manual", false);
                return;
            }

            var accessTokenAndSecret = resp["oauth_token"] + '\t' + resp["oauth_token_secret"];
            // Save new auth details
            me.removeConsumerAuthDetails("twitter");
            me.setConsumerAuthDetails("twitter", username, accessTokenAndSecret);
            // Send auth details to Twitter consumer
            me.sendAuthDetailsToConsumer("twitter");
        }

        // Replacing basic auth with xAuth
        this._consumersDetails["twitter"].service.wrappedJSObject._oauthUtils.getXAuthAccessToken({"username": username, "password": password}, onGotAccessToken);
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.nsISupports)) {
               return this;
    }

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
}
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxytunesMusicUpdatesService]);
} catch(e) {};

