/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
} catch(e) {};

var FoxyTunesJSPlayerConfig = function() {
    this.bUseChromeToContentBridge = true;
    this.bUseGlobalVolumeController = false;
    this.bUseUnicodeConverter = true;
    this.playerURLRegExp = null;
    this.playerHomepage = null;
    this.commandStrMappings = {};
    this.defaultHandlerCommands = ['Pause', 'Stop', 'Next', 'Previous', 'GetCurrentTrackTitle', 'GetCurrentTrackLength', 'GetCurrentTrackPosition', 'SetCurrentTrackPosition']; 
    this.commandsOnTimer = ['SetVolume', 'SetCurrentTrackPosition'];
    this.commandsOnTimerTimeout = 100;
};

function FoxyTunesJSPlayerBase() {
    this.commandHandlers = {};
    this.processCmdInput = {};
    this.processCmdOutput = {};
}

FoxyTunesJSPlayerBase.prototype = {

    init: function(playerConfig)
    {
    this.playerConfig = playerConfig;
    if (typeof playerConfig == 'undefined') {
        this.playerConfig = new FoxyTunesJSPlayerConfig();
    } 

    this.services = {};
    this.services['WindowMediator'] = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);
    this.services['ScriptLoader'] = Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    this.services['ObserverService'] = Components.classes["@mozilla.org/observer-service;1"]
                            .getService(Components.interfaces.nsIObserverService);
    this.services['WindowWatcher'] = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                                    .getService(Components.interfaces.nsIWindowWatcher);

    // Observing dom windows open/close
    this.services['ObserverService'].addObserver(this, "domwindowopened", false);
    this.services['ObserverService'].addObserver(this, "domwindowclosed", false);
    this.services['ObserverService'].addObserver(this, "quit-application", false);

    if (this.playerConfig['bUseChromeToContentBridge']) {
        this.loadScript("chrome://foxytunes/content/utils/chromeToContentBridge.js")
        this.chromeToContentBridge = new FoxyTunesChromeToContentBridge();
    }

    if (this.playerConfig['bUseGlobalVolumeController']) {
        this.volumeController = null;
        const kVolumeControllerCID = "@foxytunes.org/FoxyTunesEngine/FoxyTunesVolumeController;1";
        if (typeof Components.classes[kVolumeControllerCID] != 'undefined') {
            this.volumeController = Components.classes[kVolumeControllerCID]
                    .getService(Components.interfaces.IFoxyTunesVolumeController);        
        }
    }

    this.loadScript("chrome://foxytunes/content/utils/trackDataBuilder.js");

    this.commandTimers = {};

    var me = this;
    this.domContentLoadedCallback =  function(event) { me.onDOMContentLoad(event); }

    this.playersList = new Array();
    this.currentPlayer = null;
    this.lastWindowAndTab = null;

    this.createInterfaceFunctions();
    this.initCommandsOnTimer();
    this.initPlayersList();
    },

    defaultInterfaceFunctionHandler: function(cmd, value) {
         if (this.processCmdInput[cmd]) {
            value = this.processCmdInput[cmd](value);
         }
         var player = this.findPlayer();

         var cmdMapping = cmd;
         if (this.playerConfig.commandStrMappings[cmd]) {
            cmdMapping = this.playerConfig.commandStrMappings[cmd].replace("%value%", value); 
         }
         cmdHandler = this.commandHandlers[cmd] ? this.commandHandlers[cmd] : this.defaultCmdHandler;
         var res = cmdHandler.call(this, player, cmdMapping, value);

         if (this.processCmdOutput[cmd]) {
            res = this.processCmdOutput[cmd](res);
         }
         return res;
    },

    createInterfaceFunctions: function() {
        var commands = this.playerConfig.defaultHandlerCommands;

        if (!this.playerConfig['bUseGlobalVolumeController']) {
            commands.push('ToggleMute', 'IsMuted', 'GetVolume', 'SetVolume');
        }

        for (var cmd in this.playerConfig.commandStrMappings) {
            if (commands.indexOf(cmd) == -1) {
                commands.push(cmd);
            }
        }
        for (var cmd in this.commandHandlers) {
            if (commands.indexOf(cmd) == -1) {
                commands.push(cmd);
            }
        }

        var me = this;
        var createCmd = function(cmd) {
            FoxyTunesJSPlayerBase.prototype[cmd] = function(value) {
                return this.defaultInterfaceFunctionHandler(cmd, value);
            };
        };

        for (var i=0; i<commands.length; i++) {
            createCmd(commands[i]);
        }
    },

    createCommandsOnTimer: function(cmd) {
        var cmdHandler = FoxyTunesJSPlayerBase.prototype[cmd];
        if (typeof cmdHandler == 'undefined') return;
        this.commandTimers[cmd] = Components.classes["@mozilla.org/timer;1"].
						            createInstance(Components.interfaces.nsITimer); 
        FoxyTunesJSPlayerBase.prototype[cmd] = function(value) {
            var me = this;
            this.commandTimers[cmd].cancel();
            this.commandTimers[cmd].initWithCallback(function() { cmdHandler.call(me, value); }, me.playerConfig.commandsOnTimerTimeout, Components.interfaces.nsITimer.TYPE_ONE_SHOT);  
        }; 
    },

    initCommandsOnTimer: function() {
        for (var i=0;  i<this.playerConfig.commandsOnTimer.length; i++) {
            var cmd = this.playerConfig.commandsOnTimer[i];
            this.createCommandsOnTimer(cmd);
        }
    },

    Play: function() {
        if (!this.findPlayer()) {
            this.openPlayer();
            return false;
        }

        this.defaultInterfaceFunctionHandler('Play');
    },


    Pause: function() {
        if (!this.findPlayer()) {
            this.openPlayer();
            return false;
        }

        this.defaultInterfaceFunctionHandler('Pause');
    },

    ForcePause: function() {
        return this.Pause();
    },

    IsMuted : function() {
        if (this.volumeController) {
            return this.volumeController.IsMuted();
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    ToggleMute : function() {
        if (this.volumeController) {
            return this.volumeController.ToggleMute();
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    SetVolume : function(value) {
        if (this.volumeController) {
            return this.volumeController.SetVolume(value);
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    GetVolume : function() {
         if (this.volumeController) {
            return this.volumeController.GetVolume();
        } else {
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    IncreaseVolume : function() {
        var volume = this.GetVolume();
        if (volume >= 100) {
            return false;
        }
        volume = parseInt(volume) + 10;
        this.SetVolume(volume);
        return true;
    },

    DecreaseVolume : function() {
        var volume = this.GetVolume();
        if (volume <= 0) {
            return false;
        }
        volume = parseInt(volume) - 10;
        this.SetVolume(volume);
        return true;
    },

    IsRunning : function() {
        if (this.findPlayer()) return true;
        return false;
    },

    IsPlaying: function() {
        var player = this.findPlayer();
        if (!player) return false;

        return this.playerIsPlayingInDoc(player.doc);
    },

    ShowPlayer: function() {
        var player = this.findPlayer();
        if (!player) {
          this.lastWindowAndTab = null;
          this.openPlayer();
          return false;
        }
        
        var lastWindowAndTab = this.getCurrentWindowAndTab();

        if (lastWindowAndTab) {
            if (lastWindowAndTab[1] != this.currentPlayer.tab) {
                this.lastWindowAndTab = lastWindowAndTab;
            }
        } 

        var window  = player.window;
        var browser = player.browser;
        var tab     = player.tab;

        window.focus();
        browser.selectedTab = tab;

        return true;
    },

    HidePlayer: function() {
        if (!this.lastWindowAndTab) return false;

        if (!this.currentPlayer) {
            this.currentPlayer = this.findPlayer();
            if (!this.currentPlayer) return;
        }

        var currentWindowAndTab = this.getCurrentWindowAndTab(); 

        // Don't hide if we're not on the player tab
        if (currentWindowAndTab[1] != this.currentPlayer.tab) {
            return false;
        }

        var lastWindow = this.lastWindowAndTab[0];
        var lastTab = this.lastWindowAndTab[1];

        this.lastWindowAndTab = null;

        try {
            if (lastWindow) lastWindow.focus();
            if (lastWindow && lastTab) { 
                lastWindow.getBrowser().selectedTab = lastTab;
            }     
        } catch (e) {};
        return true;
    },

    IsPlayerWindowObscured: function() {
        if (!this.lastWindowAndTab) return true;
        var player = this.findPlayer();
        if (!player) return true;

        var currentWindowAndTab = this.getCurrentWindowAndTab();
        var currentTab = currentWindowAndTab[1];
        if (currentTab == player.tab) {
            return false;
        }
         
        return true;
    },

    PlayFile: function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; },
    PlayURIs: function(URIs) { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; },
    IsPlayerWindowAtForeground: function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; },
    SetOptions: function(options) { return true; },
    IsInstalled: function() { return true; },



    playerIsPlayingInDoc: function(doc) { return false; },
    defaultCmdHandler: function(player, cmd, value) {},
    newDocHandler: function(doc) {},

    playerFoundInDoc: function(doc) {
        if (!doc.location || !doc.location.href) return false;
        return doc.location.href.match(this.playerConfig['playerURLRegExp']);
    },

    findPlayer: function() {
        var playerDetails = null;

        // If last player is still playing than return it
        if (this.currentPlayer && this.currentPlayer.doc.defaultView) {

            // if the player still found in last player tab than return it if no other player is playing
            if (this.playerFoundInDoc(this.currentPlayer.doc)) {
                playerDetails = this.currentPlayer;
            }
            if (this.playerIsPlayingInDoc(this.currentPlayer.doc)) {
                    return this.currentPlayer; 
            }
        }

        for (var i=0; i<this.playersList.length; i++) {
            var currentPlayerDetails = this.playersList[i];
            
            // return the first found player tab if no previous player tab found
            if (!playerDetails) {
                playerDetails = currentPlayerDetails;
            }

            if (this.playerIsPlayingInDoc(currentPlayerDetails.doc)) {
                this.currentPlayer = currentPlayerDetails;
                return currentPlayerDetails;
            }
        }

        this.currentPlayer = playerDetails;
        return playerDetails;
    },

    initPlayersList: function() {
        var typeList = ['navigator:browser', 'foxytunes:minibrowser'];
        for (var i in typeList) {
            var windows = this.services['WindowMediator'].getEnumerator(typeList[i]);
            while (windows.hasMoreElements()) {
                var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
                // we do not want popup windows (?)
                if (!wantedWindow.toolbar.visible) continue;

                this.watchWindow(wantedWindow, true);

                var tabbrowser = wantedWindow.getBrowser();
                var tabs = tabbrowser.mTabContainer.childNodes;
                for (var iTab=0; iTab < tabs.length; iTab++) {
                    var tab = tabs[iTab];
                    var browser = tabbrowser.getBrowserForTab(tab);
                    var doc = browser.contentDocument;
                    if (!doc) continue;

                    if (!this.playerFoundInDoc(doc)) continue;

                    this.addPlayerToList(wantedWindow, doc, tab);              
                }
            }
        }
    },

    initPlayerDoc: function(doc) {
        if (this.playerConfig['bUseChromeToContentBridge']) {
            this.chromeToContentBridge.init(doc);
        }
    },

    watchWindow: function(window, bWatch) {
        if (bWatch) {
            window.addEventListener("DOMContentLoaded", this.domContentLoadedCallback, true);
        } else {
            window.removeEventListener("DOMContentLoaded", this.domContentLoadedCallback, true);
        }
    },

    getTabFromDoc: function(browser, doc) {
        var tabs = browser.tabContainer.childNodes;
        for(var i = 0; i < tabs.length; i++) {
            if (tabs[i].linkedBrowser.contentDocument == doc) {
                return tabs[i];
            }
        }
        return null;
    },

    onDOMContentLoad: function(event) {
        if (event.originalTarget.nodeName != "#document") { return; }
        var doc = event.originalTarget || document;
        if (!doc) return;
        
        var window = event.currentTarget;
        if (!window) return;
        
        this.newDocHandler(window, doc);
        if (!this.playerFoundInDoc(doc)) return;

        this.addPlayerToList(window, doc);
    },

    addPlayerToList: function(window, doc, tab) {
        if (!tab) {
            // In case of frameset use the top documet to get the tab
            var parnetDoc = doc.defaultView.frameElement ? doc.defaultView.parent.document : doc;
            tab = this.getTabFromDoc(window.getBrowser(), parnetDoc);
            if (!tab) return;
        }
        // Prevant adding the same doc twice
        for (var i=0; i<this.playersList.length; i++) {
            var player = this.playersList[i];
            if (player.doc == doc) return;
        }

        this.initPlayerDoc(doc);

         var me = this;
         doc.defaultView.addEventListener("unload", function() { me.removePlayerFromList(doc); }  , true);

         var playerDetails = new Object();
         playerDetails.window  = window;
         playerDetails.browser = window.getBrowser();
         playerDetails.tab     = tab;
         playerDetails.doc     = doc;

         this.playersList.push(playerDetails);
    },

    removePlayerFromList: function(doc) {
        var tmpList = new Array();
        for (var i=0; i<this.playersList.length; i++) {
            var player = this.playersList[i];
            if (player.doc == doc) continue;
            tmpList.push(player);
        }
        delete this.playersList;
        this.playersList = null;
        this.playersList = tmpList;
        if (this.currentPlayer && this.currentPlayer.doc == doc) this.currentPlayer = null;
    },

    getMostRecentBrowserWindow: function() {
        var win = this.services["WindowMediator"].getMostRecentWindow('navigator:browser');
        if (win && win.toolbar.visible) {
            return win;
        }

        // else
        var windows = this.services["WindowMediator"].getEnumerator('navigator:browser', true);
        while (windows.hasMoreElements()) {
            win = windows.getNext();
            if (win.toolbar.visible) return win; 
        }

        return null;
    },

    getCurrentWindowAndTab: function() {
        var lastWindow = this.services["WindowMediator"].getMostRecentWindow('navigator:browser');
        if (!lastWindow) {
            lastWindow = this.services["WindowMediator"].getMostRecentWindow('foxytunes:minibrowser');
        }
        if (!lastWindow) return null;

        var lastTabbrowser = lastWindow.getBrowser();
        var lastTab = lastTabbrowser.selectedTab; 
        var browser = lastTabbrowser.getBrowserForTab(lastTab);
        var uri = browser.currentURI.spec;
        return [lastWindow, lastTab, uri];
    },

    // nsIObserver
    observe: function(subject, topic, data) {
        switch (topic) {
            case "quit-application" :
                this.services['ObserverService'].removeObserver(this, "quit-application");
                this.services['ObserverService'].removeObserver(this, "domwindowopened");
                this.services['ObserverService'].removeObserver(this, "domwindowclosed");
                break;
            case "domwindowopened" :
                var window = subject.QueryInterface(Components.interfaces.nsIDOMWindow);
                this.watchWindow(window, true);
                break;
            case "domwindowclosed" :
                var window = subject.QueryInterface(Components.interfaces.nsIDOMWindow);
                this.watchWindow(window, false);
                break;
        }
    },

    // Player utils
    openURL: function(url) {
        var win = this.getMostRecentBrowserWindow();
        if (win) {
            var tabbrowser = win.getBrowser();
           	var newTab = tabbrowser.addTab(url);
            tabbrowser.selectedTab = newTab;
        } else {
            var array = Components.classes["@mozilla.org/supports-array;1"]
                                .createInstance(Components.interfaces.nsISupportsArray);
            var url = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
            url.data = url;

            array.AppendElement(url);
            this.services['WindowWatcher'].openWindow(windowWatcher.activeWindow,
                                 "chrome://browser/content/browser.xul",
                                 "_blank",
                                 "chrome,all,dialog=no",
                                 array);
       }
    },

    openPlayer: function() {
        if (!this.playerConfig.playerHomepage) return;
        this.openURL(this.playerConfig.playerHomepage);
    },

    loadScript: function(scriptPath) {
        this.services['ScriptLoader'].loadSubScript(scriptPath);
    },

    getTrackData: function(trackDataDetails) {
        var dataBuilder = new TrackDataBuilder();
        if (!trackDataDetails) return dataBuilder.getData();

        if (this.playerConfig.bUseUnicodeConverter) {
            var unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                getService(Components.interfaces.nsIScriptableUnicodeConverter);
            unicodeConverter.charset = 'UTF-8';
            var paramsToConvert = ["title", "artist", "album"];
            for (var i=0; i<paramsToConvert.length; i++) {
                if (trackDataDetails[paramsToConvert[i]]) {
                    trackDataDetails[paramsToConvert[i]] = unicodeConverter.ConvertFromUnicode(trackDataDetails[paramsToConvert[i]]);
                }
            }
        }

        if (trackDataDetails["title"]) {
            dataBuilder.addTitle(trackDataDetails["title"]);
        }

        if (trackDataDetails["artist"]) {
            dataBuilder.addArtist(trackDataDetails["artist"]);
        }

        if (trackDataDetails["album"]) {
            dataBuilder.addAlbum(trackDataDetails["album"]);
        }

        if (trackDataDetails["coverURI"]) {
            dataBuilder.addAlbumCoverURI(trackDataDetails["coverURI"]);
        }

        if (trackDataDetails["isVideo"]) {
            dataBuilder.addIsVideo(trackDataDetails["isVideo"]);
        }

        var displayName = '';
        if (trackDataDetails["artist"]) { 
            displayName = trackDataDetails["artist"];
            if (trackDataDetails["title"]) {
                displayName += ' - ' + trackDataDetails["title"];
            }
        } else if (trackDataDetails["title"]) {
            displayName = trackDataDetails["title"];
        }
        dataBuilder.addDisplayName(displayName);

        return dataBuilder.getData();
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    }
}
