/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/


/********* FoxyTunesLastFMController *********/

function FoxyTunesLastFMController() {
    this._streamURL = null;
    this._currentSession = null;
    this._bSupportedPlayer = false;

    this._bIsPolling = false;
    this._bPollNowPlaying = false;

    this._playerClass = "@foxytunes.org/FoxyTunesEngine/Last.fm;1";
    this._controller = null;
    
    this._volumeController = null;

    const kVolumeControllerCID = "@foxytunes.org/FoxyTunesEngine/FoxyTunesVolumeController;1";
    if (typeof Components.classes[kVolumeControllerCID] != 'undefined') {
        this._volumeController = Components.classes[kVolumeControllerCID]
                    .getService(Components.interfaces.IFoxyTunesVolumeController);        
    }

    this._windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);

    this._pollTimer = Components.classes["@mozilla.org/timer;1"].
						createInstance(Components.interfaces.nsITimer);

    this._nowPlaying = '';

    this._lastNowPlayingTime = 0;
    this._lastNowPlaying = '';

    this._initialCommandOnConnect = null;

    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/trackDataBuilder.js");
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils.js");
       scriptLoader.loadSubScript("chrome://foxytunes/content/player-config/player-config.js");
    }

    this._username = '';
    this._password = '';

    this.convertOldFormatLoginDetailsIfNeeded();

    this.wrappedJSObject = this;
}

FoxyTunesLastFMController.prototype = {
    classID : Components.ID("{1FE309A9-96BE-4243-9051-8D3877CDD735}"),

    Play : function() {
        if (this._bSupportedPlayer) {
            var me = this;
            this.startHandshake(function() { me.getNowPlayingAndStartStreamingIfNeeded(true); });
        } else {
            this.ShowPlayer();
        }
    },

    Pause : function() {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
            this._controller.Stop();
            this.handleNoStream();
        } else {
            this.ShowPlayer();
        }
    },

    ForcePause : function() {
    	if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
            this._controller.Stop();
			this.handleNoStream();
        }
    },

    Stop : function() {
    	this.Pause();
    },

    Next : function() {
        var me = this;

        function doSkip() {
            if (!me._currentSession) return;
            var url = 'http://ws.audioscrobbler.com/radio/control.php?command=skip&session=' + 
                        me._currentSession;
            me.getURL(url);
            me._nowPlaying = foxytunesGetLocaleStringExternalfunction('skipping');
            me._bPollNowPlaying = true;
        }

        if (!this._currentSession) {
            this._initialCommandOnConnect = function() { doSkip(); }
            this.startHandshake(function() { me.getNowPlayingAndStartStreamingIfNeeded(true); });
            return;
        } else {
            doSkip();
        }
    },

    Previous : function() {
    	throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    ToggleMute : function() {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
        	this._controller.ToggleMute();
        } else {
            if (this._volumeController) return this._volumeController.ToggleMute();
        }
    },

    SetVolume : function(value) {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
    	    this._controller.SetVolume(value);
        } else {
            if (this._volumeController) return this._volumeController.SetVolume(value);
        }
    },

    GetVolume : function() {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
            return this._controller.GetVolume(); 
        } else {
            if (this._volumeController) {
                return this._volumeController.GetVolume();
            } else {
                return 50;
            }
        }
    },

    IncreaseVolume : function() {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
            this._controller.IncreaseVolume();
        } else {
            if (this._volumeController) {
                 var volume = this._volumeController.GetVolume();
                 var newVolume = Math.min(volume + 10, 100);
                 this._volumeController.SetVolume(newVolume);
            }
        }
    },

    DecreaseVolume : function() {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
        	this._controller.DecreaseVolume();
        } else {
            if (this._volumeController) {
                 var volume = this._volumeController.GetVolume();
                 var newVolume = Math.max(volume - 10, 0);
                 this._volumeController.SetVolume(newVolume);
            }
        }
    },

    GetCurrentTrackTitle : function() {
        if (this._bIsPolling && !this._bPollNowPlaying) return this._nowPlaying;

        if (this._currentSession == 'FAILED') return this._nowPlaying;
    
        var date = new Date();
        var now = date.getTime();
        if ((this._lastNowPlayingTime !=0 ) && ((now - this._lastNowPlayingTime) < 10000)) return this._nowPlaying;

        if (this._currentSession) {
            // no session yet, don\'t update time
            this._lastNowPlayingTime = now;
        }

        if (!this._currentSession) {
            var me = this;
            this.startHandshake(function() { me.getNowPlayingAndStartStreamingIfNeeded(false); });
            return this._nowPlaying;
        }

        var url = this.buildNowPlayingURL(); 
        if (!url) return this._nowPlaying; 

        var me = this;
        this.getURL(url, function(data) { me.parseAndSetNowPlaying(data); });
        return this._nowPlaying;
    },

    GetCurrentTrackLength : function() {
       return 0;
    },

    GetCurrentTrackPosition : function() {
       return 0;
    },

    ShowPlayer : function() {
        var bUsingWebPlayer = this.showWebplayer();
        if (bUsingWebPlayer) return true;

        this.initControllerIfNeeded();
        if (!this._controller.ShowPlayer()) {
            this.openLastFm();
        }

        return true;
    },

    HidePlayer : function() {
        var bUsingWebPlayer = this.hideWebPlayer();
        if (!bUsingWebPlayer) {
            this.initControllerIfNeeded();
            return this._controller.HidePlayer();
        }
        return true;
    },

    PlayFile : function() {
        if (this._bSupportedPlayer) {
            // TBD
        } else {
            this.ShowPlayer();
        }
    },

    SetOptions  : function(options) {
        var playerClass = null;
        this._currentSession = null;
        var optionsArr = options.split(' ');
        for (var i=0; i < optionsArr.length; i++) {
            if (optionsArr[i] == '-username') {
                try {
                    this._username = optionsArr[i+1];
                } catch(e) {
                    dump('Error: couldn\'t get last.fm username from command line\n');
                }
            }

            if (optionsArr[i] == '-password') {
                try {
                    this._password = optionsArr[i+1];
                } catch(e) {
                    dump('Error: couldn\'t get last.fm password from command line\n');
                }
            }

            if (optionsArr[i] == "-playerclass") {
                try {
                    playerClass = optionsArr[i+1];
                } catch(e) {
                    dump('Error: couldn\'t get last.fm player class from command line\n');
                }
            }
        }
        
        this._controller = null;
        if (playerClass != "null") {
            this._playerClass = playerClass;
        }

        if (playerClass != "null" && playerClass != "@foxytunes.org/FoxyTunesEngine/Last.fm;1") {
            this._bSupportedPlayer = true;
        } else {
            this._bSupportedPlayer = false;
        }

        return true;
    },

    SetCurrentTrackPosition : function(pos) {
    },

    IsMuted : function() {
        if (this._bSupportedPlayer) {
            this.initControllerIfNeeded();
            return this._controller.IsMuted();
        } else {
    	    if (this._volumeController) return this._volumeController.IsMuted();
        }
    },

    PlayURIs : function(URIs) {
        var me = this;

        function doPlayURIs(URIs) {
            if (!me._currentSession) return false;
            var url = 'http://ws.audioscrobbler.com/radio/adjust.php?url=' + URIs + '&session=' +
                        me._currentSession;
            me.getURL(url, function(data) {me.getNowPlayingAndStartStreamingIfNeeded('true');});
            me._nowPlaying = foxytunesGetLocaleStringExternalfunction('loading');
            me._bPollNowPlaying = true;
        }

        if (!this._currentSession) {
            this.startHandshake( function() {doPlayURIs(URIs, true);});
            return;
        } else {
            doPlayURIs(URIs);
        }
    },

    IsInstalled : function() {
        if (this.getPlatform() != 'mac') {
            this.initControllerIfNeeded();
        return this._controller.IsInstalled();  
        } else {
            // Problem with binding initing last.fm controller on startup on Mac
            // causes a 'choose application' dialog to popup
            throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
        }
    },

    IsRunning : function() {
        var lastFmWebPlayerWindow = this.findWebPlayerWindow();
        if (lastFmWebPlayerWindow) {    
            return true;
        } else {
            var isInstalled = this.IsInstalled();
            if (!isInstalled) return false;

            this.initControllerIfNeeded();
            return this._controller.IsRunning();
        }
    },

    IsPlaying: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED; // not implementing because we have problems supporting Pause so I prefer showing both play and pause buttons
    },

    IsPlayerWindowObscured: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        var lastFmWebPlayerWindow = this.findWebPlayerWindow();
        if (!lastFmWebPlayerWindow) {
            this.initControllerIfNeeded();
            return this._controller.IsPlayerWindowObscured();
        }
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsPlayerWindowAtForeground: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;

        var lastFmWebPlayerWindow = this.findWebPlayerWindow();
        if (!lastFmWebPlayerWindow) {
            this.initControllerIfNeeded();
            return this._controller.IsPlayerWindowAtForeground();
        }
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    // private
    getURL : function(url, callback) {
        if (!url) return false;
        var xmlHTTPrequest = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].
                                    createInstance(Components.interfaces.nsIXMLHttpRequest);
        xmlHTTPrequest.open('get', url, true);
        var me = this;
        xmlHTTPrequest.onreadystatechange = function() {
                if (xmlHTTPrequest.readyState != 4) return;
                if (xmlHTTPrequest.status != 200) return;
                var contents = xmlHTTPrequest.responseText;
                if (callback) {
                    callback(contents);
                }
        }
        xmlHTTPrequest.send(null);
    },

    buildNowPlayingURL: function() {
        if (!this._currentSession) return null;
        var url = 'http://ws.audioscrobbler.com/radio/np.php?session=' + this._currentSession;
        return url;
    },

    startHandshake: function (callback) {
        var username = this._username;
        var password = this._password;
        if (!username || !password) return;

        var url = 'http://ws.audioscrobbler.com/radio/handshake.php' + 
	                    '?username=' + username +  
          	            '&passwordmd5=' + password;

        this._nowPlaying = foxytunesGetLocaleStringExternalfunction('connecting');
        var me = this;
        this.getURL(url, function(data) { me.handShakeDone(data, callback); });
    },

    parseReply : function(data) {
        var lines = data.split('\n');
        var params = {};
        for (var i=0; i < lines.length; i++) {
            var line = lines[i];
            var parts = line.split('=');
            if (parts.length >= 2) {
                var name = parts.shift();
                var value = parts.join('=');
                params[name] = value;
            }
        }
        return params;
    },

    handShakeDone : function(data, callback) {
        var params = this.parseReply(data);

        if (params['session'] == 'FAILED') {
            var msg = params['msg'];
            this._nowPlaying = foxytunesGetLocaleStringExternalfunction('errorWithString', ['"' + msg + '"']);
            this._currentSession = 'FAILED';
            return;
        }

        this._currentSession = typeof params['session'] != 'undefined' ? params['session'] : null;
        this._streamURL = params['stream_url'];
        if (callback) {
            callback();
        }
    },

    getNowPlayingAndStartStreamingIfNeeded : function(bLaunch) {
         if (this._currentSession) {
            var url = this.buildNowPlayingURL(); 
            if (!url) return ''; 
            var me = this;
            this.getURL(url, function(data) { me.parseNowPlayingAndStartStreamingIfNeeded(data, bLaunch); });        
        }
    },

    parseNowPlayingAndStartStreamingIfNeeded : function(data, bLaunch) {
        var params = this.parseReply(data);
        if (typeof params['ERROR'] != 'undefined') {
            // Error encountered
            this._currentSession = null;
            return; 
        }

        if (typeof params['streaming'] == 'undefined') return; // we should always get at least that

        if (params['streaming'] == 'false') {
            // Stop polling is no stream found
            this._nowPlaying = '';
            this.handleNoStream();
            if (bLaunch && this._streamURL) {
                if (this._bSupportedPlayer) {
                    this.initControllerIfNeeded();
                    this._controller.PlayURIs(this._streamURL);
                }
                this._bPollNowPlaying = true;
            }
        } else {
            this.parseAndSetNowPlaying(data);
            if (this._initialCommandOnConnect) {
                this._initialCommandOnConnect();
                this._initialCommandOnConnect = null;
            }
        }
    },

    parseAndSetNowPlaying : function(data) {
        var params = this.parseReply(data);
        if (typeof params['ERROR'] != 'undefined') {
            // Error encountered
            this._currentSession = null;
            return ''; 
        }
        
        if (params['streaming'] == 'false') {
            // Stop polling is no stream found
            this.handleNoStream();  
            this._nowPlaying = '';
            return '';
        }

        var unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                getService(Components.interfaces.nsIScriptableUnicodeConverter);
        unicodeConverter.charset = 'UTF-8';

        var dataBuilder = new TrackDataBuilder();
        if (params.artist) dataBuilder.addArtist(unicodeConverter.ConvertFromUnicode(params.artist));
        if (params.track) dataBuilder.addTitle(unicodeConverter.ConvertFromUnicode(params.track));
        if (params.album) dataBuilder.addAlbum(unicodeConverter.ConvertFromUnicode(params.album));

        var displayName = '';
        if (params.artist) { 
            displayName = params.artist;
            if (params.track) {
                displayName += ' - ' + params.track;
            }
        } else if (params.track) {
            displayName = params.track;
        }
        dataBuilder.addDisplayName(unicodeConverter.ConvertFromUnicode(displayName));

        if (params['albumcover_small']) {
            var coverURL = params['albumcover_small'];
            if (!coverURL.match(/static.last.fm.*\/noimage\//)) {
                dataBuilder.addAlbumCoverURI(coverURL);
            }
        }
        dataBuilder.addIsVideo("no");

        this._nowPlaying = dataBuilder.getData();

        if (this._nowPlaying != this._lastNowPlaying) {
            if (!this._bPollNowPlaying) {
                this._bPollNowPlaying = true;
            } else {
                if (params.trackduration) {
                    // New song - stop polling until 20 secs before the song end
                    this._bPollNowPlaying = false;
                    var delay = params.trackduration - 20;
                    if (delay < 0) delay = 0;
                    this._pollTimer.initWithCallback(this, 
                                     delay * 1000,
                                     Components.interfaces.nsITimer.TYPE_ONE_SHOT);
                }
            }
        } 

        this._lastNowPlaying = this._nowPlaying;
    },

    notify: function(aTimer) {
        if (aTimer == this._pollTimer) {
            var url = this.buildNowPlayingURL(); 
            if (!url) return;
            var me = this;
            this.getURL(url, function(data) { me.parseAndSetNowPlaying(data); me._bPollNowPlaying = true; });
        }
    },

    handleNoStream: function() {
        this._bPollNowPlaying = false;
        this._pollTimer.cancel();
    },

    showWebplayer: function() {
        var lastFmWebPlayerWindow = this.findWebPlayerWindow();
        if (lastFmWebPlayerWindow) {
            lastFmWebPlayerWindow.focus();
            return true;
        } 
        return false;
    },

    hideWebPlayer: function() {
        var lastFmWebPlayerWindow = this.findWebPlayerWindow();
        if (lastFmWebPlayerWindow) {
            lastFmWebPlayerWindow.minimize();
            return true;
        }
        return false;
    },

    findWebPlayerWindow: function() {
        var windows = this._windowMediator.getEnumerator('navigator:browser');
        while (windows.hasMoreElements()) {
            var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
            // only search popup windows
            if (!wantedWindow.toolbar.visible &&
                  wantedWindow.document.title.match(/last\.?fm/i)) {
                    if (wantedWindow.content && wantedWindow.content.document) {
                        if (wantedWindow.content.document.location.toString().match(/webclient/i)) {
                            return wantedWindow;
                        }
                    }
            }
        }
        return null;    
    },

    getMostRecentBrowserWindow : function() {
        var win = this._windowMediator.getMostRecentWindow('navigator:browser');
        if (win && win.toolbar.visible) {
            return win;
        }

        // else
        var windows = this._windowMediator.getEnumerator('navigator:browser', true);
        while (windows.hasMoreElements()) {
           win = windows.getNext();
           if (win.toolbar.visible) return win; 
        }

        return null;
    },

    openLastFm: function() {
        const lastFmURL = 'http://www.last.fm';
        var win = this.getMostRecentBrowserWindow();
        if (win) {
                var tabbrowser = win.getBrowser();

           		var newTab = tabbrowser.addTab(lastFmURL);
                tabbrowser.selectedTab = newTab;
        } else {
            var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                                    .getService(Components.interfaces.nsIWindowWatcher);
            var array = Components.classes["@mozilla.org/supports-array;1"]
                                .createInstance(Components.interfaces.nsISupportsArray);
            var url = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
            url.data = lastFmURL;

            array.AppendElement(url);
            windowWatcher.openWindow(windowWatcher.activeWindow,
                                 "chrome://browser/content/browser.xul",
                                 "_blank",
                                 "chrome,all,dialog=no",
                                 array);
       }
    },

    getPlatform: function() {
        var win = this.getMostRecentBrowserWindow();
        var platform = win.navigator.platform.toLowerCase();
	
	    if (platform.indexOf('linux') != -1) {
		    return 'linux';
	    }

	    if (platform.indexOf('mac') != -1) {
		    return 'mac';
	    }

        return "windows";
    },

    initControllerIfNeeded: function() {
        if (this._controller != null) return;
        try {
            this._controller = Components.classes[this._playerClass].
                            getService(Components.interfaces.mozIFoxyTunes);
        } catch(e) {};
    },

    convertOldFormatLoginDetailsIfNeeded: function() {
        var playerName = "last.fm";
        var oldFormatPassword = gFTPlayerConfig.getPlayerConfigVariable(playerName, 'password');
        // User is using old format - convert to new one
        if (oldFormatPassword && oldFormatPassword != "") {
            var oldFormatUsername = gFTPlayerConfig.getPlayerConfigVariable(playerName, 'username');
            try { 
                gFTPlayerConfig.clearPlayerConfigVariable(playerName, 'password');
                gFTPlayerConfig.clearPlayerConfigVariable(playerName, 'username');
            } catch(e) {};
            gFTPlayerConfig.setPlayerLoginDetails(playerName, oldFormatUsername, oldFormatPassword);
        }
    },

    // nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    }
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesLastFMController]);
} catch(e) {};
