/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/
var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);

if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes-components-js/content/players/FoxyTunesJSPlayerBase.js");
}

var playerConfig = new FoxyTunesJSPlayerConfig();
playerConfig.playerURLRegExp = new RegExp("rhapsody.com");
playerConfig.playerHomepage = "http://www.rhapsody.com/";
playerConfig.commandStrMappings["Play"] = "doPlay();";
playerConfig.commandStrMappings["Pause"] = "doPause();";
playerConfig.commandStrMappings["Next"] = "doPlayerNext();";
playerConfig.commandStrMappings["Previous"] = "doPlayerPrevious();";
playerConfig.commandStrMappings["GetCurrentTrackPosition"] = "getPosition();";
playerConfig.commandStrMappings["SetCurrentTrackPosition"] = "setPosition(%value%);";
playerConfig.commandStrMappings["SetVolume"] = "setVolume(%value%);";
playerConfig.commandStrMappings["GetVolume"] = "getVolume();";
playerConfig.commandStrMappings["Stop"] = "doStop();";
playerConfig.commandStrMappings["IsMuted"] = "getMute();";
playerConfig.commandStrMappings["GetCurrentTrackLength"] = "getTrackLength();";

function FoxyTunesRhapsodyController() {
    this.init(playerConfig);
}

FoxyTunesRhapsodyController.prototype = new FoxyTunesJSPlayerBase();
FoxyTunesRhapsodyController.prototype.classID = Components.ID("{A4B65999-9849-4206-A9D4-180039232132}");

FoxyTunesRhapsodyController.prototype.processCmdInput["SetVolume"] = function(value) { return value/100; };
FoxyTunesRhapsodyController.prototype.processCmdInput["SetCurrentTrackPosition"] = function(value) { return (value*this.GetCurrentTrackLength())/100; };
FoxyTunesRhapsodyController.prototype.processCmdOutput["GetVolume"] = function(value) { return Math.round(value*100); };
FoxyTunesRhapsodyController.prototype.processCmdOutput["IsMuted"] = function(value) { return (value == "true") };
FoxyTunesRhapsodyController.prototype.processCmdOutput["GetCurrentTrackLength"] = function(value) { return value/1000 };
FoxyTunesRhapsodyController.prototype.processCmdOutput["GetCurrentTrackPosition"] = function(value) { return value/1000 };

FoxyTunesRhapsodyController.prototype.defaultCmdHandler = function(player, cmd, value) {
    if (!player) return false;
    return this.sendPlayerCmd(player.doc, cmd);
};

FoxyTunesRhapsodyController.prototype.commandHandlers["ToggleMute"] = function(player, cmd, value) {
    return this.sendPlayerCmd(player.doc, 'setMute(' + !this.IsMuted() + ')');
};

FoxyTunesRhapsodyController.prototype.commandHandlers["GetCurrentTrackTitle"] = function(player, cmd, value) {
    if (!player) return this.getTrackData();

    var artist = this.sendPlayerCmd(player.doc, 'getArtistName()');
    var album = this.sendPlayerCmd(player.doc, 'getAlbumName()');
    var title = this.sendPlayerCmd(player.doc, 'getTrackName()');

    return this.getTrackData({"title": title, "artist": artist, "album": album, "isVideo": "no"});
};

FoxyTunesRhapsodyController.prototype.playerIsPlayingInDoc = function(doc) {
    return (this.sendPlayerCmd(doc, "getPlayState()") == "4");
};

FoxyTunesRhapsodyController.prototype.playerFoundInDoc = function(doc) {
    if (!doc.location || !doc.location.href) return false;
    if (!doc.location.href.match(this.playerConfig['playerURLRegExp'])) return false;
    // In case of a popup player
    if (doc.location.href.match('http://www.rhapsody.com/player')) return true;

    return (doc.getElementById('popoutPlayer_embedded') != null);
};

FoxyTunesRhapsodyController.prototype.sendPlayerCmd = function(doc, cmd) {
    var rhapCmd = '';
    rhapCmd += 'function() {';
    rhapCmd += '    var playerType = document.getElementById("popoutPlayer_embedded") ? "embedded" : "popout";';
    rhapCmd += '    return RhapsodyPlayer.getFlashApp(playerType).' + cmd + ';';
    rhapCmd += '}()';

    return this.chromeToContentBridge.fireCommand(doc, rhapCmd);
};

try {
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesRhapsodyController]);
} catch(e) {};
