/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

const MIN_INTERVAL_BETWEEN_HISTORY_TRACKS = 20; 
const Cc = Components.classes;
const Ci = Components.interfaces;

var gUnicodeConverter = null;

var TrackDataUtils = {
    newTrackData : function() {
        return new FoxyTunesTrackData();
    },

    normalize : function(trackData) {
        for (var item in trackData) {
            if (typeof trackData[item] == 'string') {
                trackData[item] = TrackDataUtils.trim(trackData[item]);
            }
        }
        if (trackData.location) {
            trackData.location = trackData.location.replace(/\\/g, '/');
        }

    },

    trim : function(str) {
        str = str.replace(/^ +/, '');
        str = str.replace(/ +$/, '');
        return str;
    },

    areEqual : function(trackData1, trackData2) {
        // do not compare by album (because that can be filled later)
        // TBD - better history update mechanism
        if (
            (trackData1.artist == trackData2.artist) && 
            (trackData1.title  == trackData2.title) &&
            (trackData1.originalTitle  == trackData2.originalTitle) 
           ) {
            //dump('\n\neq: ' + trackData1 + ' with ' + trackData2 + '\n\n');
            return true;
        }
        //dump('\n\nNOT !!! eq: ' + trackData1 + ' with ' + trackData2 + '\n\n');
        return false;
    },

    toUnicode : function(trackData) {
        if (!gUnicodeConverter) {
            gUnicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                    createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
        }

        gUnicodeConverter.charset = 'UTF-8';
        var unicodeTrackData = {};

        for (var i in trackData) {
            if (i == 'QueryInterface') continue;
            var val = trackData[i];
            if (typeof val != 'string') {
                unicodeTrackData[i] = val;
            } else {
                try {
                    unicodeTrackData[i] = gUnicodeConverter.ConvertToUnicode(val);
                } catch(e) {
                    //dump("Problems with unicode convertion\n");
                    unicodeTrackData[i] = val;
                }
            }
        }

        unicodeTrackData.wrappedJSObject = unicodeTrackData;
        return unicodeTrackData;
    },

}

function PlayBackHistory() {
    FTJSLib.include(FTJSLib.jslib_file);

    this._historyTrackList = new FoxyTunesTrackList();

    this._lastTrackIdAdded = 0;

    this._observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);
}

PlayBackHistory.prototype = {

    addTrack : function(trackData) {
        var unicodeTrackData = TrackDataUtils.toUnicode(trackData);
        var prevTrackData = this._historyTrackList.length > 0 ? 
                                  this._historyTrackList[this._historyTrackList.length-1] :
                                  TrackDataUtils.newTrackData();
        if (unicodeTrackData.originalTitle && !TrackDataUtils.areEqual(prevTrackData, unicodeTrackData)) {
            var deltaFromPrevMilli = unicodeTrackData.playTime - prevTrackData.playTime;
            var deltaFromPrevSec = deltaFromPrevMilli / 1000;
            if ((deltaFromPrevSec > MIN_INTERVAL_BETWEEN_HISTORY_TRACKS) || (this._historyTrackList.length == 0)) {
                // add this track
                this._historyTrackList.push(unicodeTrackData);
                this._observerService.notifyObservers(null, "foxytunes-internal-playback-history", "track-added-internal");        
            } else {
                // the previous track skipped - replace it with this one
                this._historyTrackList[this._historyTrackList.length-1] = unicodeTrackData;
                this._observerService.notifyObservers(null, "foxytunes-internal-playback-history", "track-skipped-internal");
            }


            this._lastTrackIdAdded = unicodeTrackData.id;
        }

        if (unicodeTrackData.originalTitle && TrackDataUtils.areEqual(prevTrackData, unicodeTrackData)) {
            // update the start time in the current track
            unicodeTrackData.id = prevTrackData.id;
            unicodeTrackData.playTime = prevTrackData.playTime;

            // update the album if needed in the current track
            // scenario - we don't have an album, but an external resolver 
            // fetched one for us
            if (!unicodeTrackData.album && prevTrackData.album) {
                unicodeTrackData.album = prevTrackData.album;
            }
        }
    },

    removeTrack : function(id) {
        
    },

    updateTrack : function(id, trackData) {
    },

    getTracks : function(nTracks) {
        return this._historyTrackList;
    }
}

function FoxyTunesTrackList() {
    this.wrappedJSObject = this;

    this._unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                createInstance(Components.interfaces.nsIScriptableUnicodeConverter);

}

FoxyTunesTrackList.prototype = new Array;

function FoxyTunesTrackData() {
    this.wrappedJSObject = this;

    // milliseconds since 1970
    this.playTime   = new Date();
    this.playTime = this.playTime.getTime();

    // last known playback position 
    // this probably shouldn't be here
    // as it reflects the state of the player
    // rather than the track data
    // but its convenient for clients that
    // want to know the current track status
    this.position   = null;

    // probably needs to be changed
    this.id = this.playTime;
   
    this.player = null;

    this.aatWasGuessed = true;

    this.artist = null;
    this.album  = null;
    this.title  = null;
    this.location    = null;
    this.albumCoverURL = null;
    
    this.genre    = null;
    this.year     = null;
    this.length   = null;

    this.rating = 0;

    this.originalTitle = '';
    this.normalizedOriginalTitle = '';

    this.isVideo   = 'unknown';
}

FoxyTunesTrackData.prototype = {
    QueryInterface: function (iid) {
    	if (iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    },

    toString : function() {
        var val = '';
        for (var item in this) {
            if (typeof this[item] == 'string') {
            }
        }
        return val;
    },
}

// Checking for signature sites xml update
// in the future we can refactore this class to provide
// generic file update mechanism
FoxyTunesSignatureSitesUpdateService = function() {
    this.url = 'https://updates.foxytunes.com/firefox/signatunes/site-support/?item_version=%ITEM_VERSION%&foxytunes_version=%FT_VERSION%&app_id=%APP_ID%&app_version=%APP_VERSION%&app_os=%APP_OS%&app_abi=%APP_ABI%';
    this.observerService = Components.classes["@mozilla.org/observer-service;1"]
                                .getService(Components.interfaces.nsIObserverService);
    this.prefService = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);
    this.prefBranch  = this.prefService.getBranch("foxytunes.");
    this.updateInterval = 86400;
    try {
        this.updateInterval = this.prefBranch.getIntPref('signatures.update.interval');
    } catch (e) {};
},

FoxyTunesSignatureSitesUpdateService.prototype = {
    init: function() {
        var updateTimerManager = Components.classes["@mozilla.org/updates/timer-manager;1"]
                                   .getService(Components.interfaces.nsIUpdateTimerManager);
        updateTimerManager.registerTimer("foxytunes-signature-sites-xml-update-timer", this, this.updateInterval);
    },

    writeFile: function(text) {
        var sitesXML = Components.classes["@mozilla.org/file/directory_service;1"].
                            getService(Components.interfaces.nsIProperties).
                            get("ProfD", Components.interfaces.nsILocalFile);
        sitesXML.append("FoxyTunes");
        sitesXML.append("signatureSites.xml");
        var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                              createInstance( Components.interfaces.nsIFileOutputStream );
        outputStream.init(sitesXML, 0x04 | 0x08 | 0x20,  0777, 0);
    	outputStream.write(text, text.length);
	    outputStream.close();
    },

    checkForUpdates: function(params) {
        this.notify(null, params);
    },

    buildUpdateURL: function(foxytunesVersion, signatunesXMLVersion, initialUpdateCheck) {
        var url = this.url;

         var app = Components.classes["@mozilla.org/xre/app-info;1"]
                   .getService(Components.interfaces.nsIXULAppInfo)
                   .QueryInterface(Components.interfaces.nsIXULRuntime);

        var version = signatunesXMLVersion;
        if (!version) {
            version = '---';
        try {
            version = this.prefBranch.getCharPref('signatures.sitelist.version');
        } catch (e) {};
        if (version == '---') return null;
        } 

        var ftversion = foxytunesVersion;
        if (!ftversion) {
            var ftversion = '-';
            try {
                ftversion = this.prefBranch.getCharPref('version');
            } catch (e) {};
        } 

        url = url.replace(/%FT_VERSION%/, ftversion);
        url = url.replace(/%ITEM_VERSION%/, version);
        url = url.replace(/%APP_ID%/, app.ID);
        url = url.replace(/%APP_VERSION%/, app.version);
        url = url.replace(/%APP_OS%/, app.OS);
        url = url.replace(/%APP_ABI%/, app.XPCOMABI);

        if (initialUpdateCheck) {
            url += '&initial_update=yes';
        }

        var isInternationalVersion = false;
        try {
            isInternationalVersion = this.prefBranch.getBoolPref('international');
        } catch (e) {};        

        if (isInternationalVersion) {
            url += '&international=yes';
        }

        return url;
    },

    notify: function(aTimer, params) {
        if (typeof params == 'undefined') params = null;
        var callback = (params && params.callback) ? params.callback : null;
        var foxytunesVersion = (params && params.foxytunesVersion) ? params.foxytunesVersion : null;
        var signatunesXMLVersion = (params && params.signatunesXMLVersion) ? params.signatunesXMLVersion : null;
        var initialUpdateCheck = (params && params.initialUpdateCheck) ? params.initialUpdateCheck : null;

        // Ignore auto-update and disable if has callback
        if (!callback) {
            var autoUpdateSiteList = true;
            try {
                autoUpdateSiteList = this.prefBranch.getBoolPref('signatures.sitelist.auto-update');
            } catch (e) {};
            var signatunesDisabled = 'false';
            try {
                signatunesDisabled = this.prefBranch.getCharPref('signatures.disable');
            } catch (e) {};
            if ((autoUpdateSiteList != true) || (signatunesDisabled == 'true')) return;
        }

        var url = this.buildUpdateURL(foxytunesVersion, signatunesXMLVersion, initialUpdateCheck);
        if (callback) url = url + '&force=yes';
        if( !url) return;
        // dump('singatunes - checking for updates...' + url + '\n');

        var me = this;
        var xmlHTTPrequest = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].
                                    createInstance(Components.interfaces.nsIXMLHttpRequest);
        xmlHTTPrequest.open('get', url, true); 
        xmlHTTPrequest.onreadystatechange = function() {
            if (xmlHTTPrequest.readyState == 4) {
                // for bug in mozilla that throw error when calling status
            	var status =  -1;
		        try {
                    status = xmlHTTPrequest.status;
            	} catch (e) {}
                if (status == 200) {
                    var bUpdateFound = false;
    	  	        var doc = xmlHTTPrequest.responseXML;
                    if (doc.getElementsByTagName('signature-sites')[0]) {
                        me.writeFile(xmlHTTPrequest.responseText);
                        me.observerService.notifyObservers(null, "foxytunes-internal-signatures", "foxytunes-signatures-sites-xml-updated");
                        // dump('updated sites xml\n');
                        bUpdateFound = true;
                    } else {
                        // dump('No update needed!\n');
                    }
                    if (callback) callback.notify(bUpdateFound);
                } 
           }
       }
       xmlHTTPrequest.send(null);
    }
}

/********* FoxyTunesService *********/

function FoxyTunesService() {
    // preferences
    this._prefService = Components.classes["@mozilla.org/preferences-service;1"].
                            getService(Components.interfaces.nsIPrefService);
    this._prefBranch  = this._prefService.getBranch("foxytunes.");
    this._prefBranch2 = this._prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this._prefBranch2.addObserver("", this, false); 
    this._prefs = new Object();
    this.readPrefs();

    // DOM parser
    this._DOMParser = Components.classes["@mozilla.org/xmlextras/domparser;1"].
							createInstance(Components.interfaces.nsIDOMParser);

    // Unicode converter
    this._unicodeConverter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
                                createInstance(Components.interfaces.nsIScriptableUnicodeConverter);

    // controller
    this._controller = null;

    // Always keep auto switch players alive
    this.initAutoSwitchPlayers();
    this._allowSwitchBack = false;

    // current track title (raw title xml returned from player)
    this._currentTrackTitle = null;

    // a flag that marks the need to send a 'track changed' notification
    this._bCurrentTrackTitleIsDirty = false;

    // current track data
    this._currentTrackData = TrackDataUtils.newTrackData();

    // last known track length and position
    this._currentTrackLength = 0;
    this._currentTrackPosition = 0;

     this.initPlayerPropStates();

    // poll timer
    this._pollTimer = Components.classes["@mozilla.org/timer;1"].
						createInstance(Components.interfaces.nsITimer);
    this._firstTimerSwitchBack = true;

    if (this._prefs.trackDataPollInterval > 0) { 
         this._pollTimer.initWithCallback(this, 
                                     this._prefs.trackDataPollInterval,    
                                     Components.interfaces.nsITimer.TYPE_REPEATING_SLACK);
    }
    
    // observer service
	this._observerService = Components.classes["@mozilla.org/observer-service;1"].
                                getService(Components.interfaces.nsIObserverService);
    this._observerService.addObserver(this, "quit-application", false);
    this._observerService.addObserver(this, "foxytunes-poll-player-state-internal", false);

    // get ftlib (and jslib)
    Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
       getService(Components.interfaces.mozIJSSubScriptLoader).
       loadSubScript('chrome://foxytunes/content/ftlib/ftlib.js');

    // history
    this._playbackHistory = new PlayBackHistory();

    // Signatures xml update
    try {
        this.foxyTunesSignatureSitesUpdateService = new FoxyTunesSignatureSitesUpdateService();
        this.foxyTunesSignatureSitesUpdateService.init();
    } catch(e) {
        dump("Error - can't init foxyTunesSignatureSitesUpdateService (" + e + ")\n");
    }
}

FoxyTunesService.prototype = {
    // mozIFoxyTunes
    classID: Components.ID("{8AE88EE0-C1A6-4691-B1CC-6370F5F5E158}"),

    Play : function() {
        this.getControllerIfNeeded();
        try {
            var rc = this._controller.Play();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    Pause : function() {
        this.getControllerIfNeeded();
        try {
           var rc =  this._controller.Pause();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    ForcePause : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.ForcePause();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    Stop : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.Stop();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    Next : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.Next();
        } catch(e) {
            throw e;
        }
        return rc;
    },            

    Previous : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.Previous();
        } catch(e) {
            throw e;
        }
        return rc;
    },


    ToggleMute : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.ToggleMute();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    SetVolume : function(value) {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.SetVolume(value);
        } catch(e) {
            throw e;
        }
        return rc;
    },

    GetVolume : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.GetVolume();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    IncreaseVolume : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.IncreaseVolume();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    DecreaseVolume : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.DecreaseVolume();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    GetCurrentTrackTitle : function() {
        this.getControllerIfNeeded();
        var title = null;        
        try {
            title = this._controller.GetCurrentTrackTitle();
        } catch(e) {
            throw e;
        }

        if (title != this._currentTrackTitle) {
            this._bCurrentTrackTitleIsDirty = true;
        }

        this._currentTrackTitle = title;

        if (!title) {
            this._currentTrackData = TrackDataUtils.newTrackData();
            return null;
        }

        if (this._prefs.bURLUnescape) {
               try {
                   title = unescape(title);
               } catch(e) {}
        }
        
        try {
    		if (this._prefs.characterEncoding != 'default') {            
	    		title = this.charsetStringToUTF8(title, this._prefs.characterEncoding);
    		} else {
	    		title = this.charsetStringToUTF8(title, 'UTF-8');
    		}
        } catch(e) {}

        this._currentTrackData = TrackDataUtils.newTrackData();
        this._currentTrackData.player = this._prefs.playerClass;
        if (title.substr(0, 7) == '<track>') {
			this.getTrackDataFromXML(title);
	    } else {
            this._currentTrackData.originalTitle = title;
            this._currentTrackData.normalizedOriginalTitle = title;
        }

        this.fixCurrentTrackData();

        this._playbackHistory.addTrack(this._currentTrackData);

        return this._currentTrackData.originalTitle;
    },

    GetCurrentTrackLength : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.GetCurrentTrackLength();
        } catch(e) {
        //    throw e;
        }

        this._currentTrackLength = rc;
        return rc;
    },

    GetCurrentTrackPosition : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.GetCurrentTrackPosition();
        } catch(e) {
        //    throw e;
        }

        this._currentTrackPosition = rc;
        return rc;
    },

    ShowPlayer : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.ShowPlayer();
        } catch(e) {
            throw e;
        }

        this._playerPropState['showhide'].state = !this._playerPropState['showhide'].state;

        return rc;
    },

    HidePlayer : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.HidePlayer();
        } catch(e) {
            throw e;
        }

        this._playerPropState['showhide'].state = !this._playerPropState['showhide'].state;

        return rc;
    },

    PlayFile : function() {
        this.getControllerIfNeeded();
        try {
           var rc = this._controller.PlayFile();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    SetOptions  : function(options) {
        this.getControllerIfNeeded();
        try {
            var rc = this._controller.SetOptions(options);
        } catch(e) {
            throw e;
        }
        return rc;
    },

    SetCurrentTrackPosition : function(pos) {
        this.getControllerIfNeeded();
        try {
            var rc = this._controller.SetCurrentTrackPosition(pos);
        } catch(e) {
            throw e;
        }
        return rc;
    },

    IsMuted : function() {
        this.getControllerIfNeeded();
        try {
            var rc = this._controller.IsMuted();
        } catch(e) {
            throw e;
        }

        return rc;
    },


    PlayURIs : function(URIs) {
        this.getControllerIfNeeded();

        var bPlayed = this.playiTunesM3UIfNeeded(URIs);
        if (bPlayed) return;

        try {
            var rc = this._controller.PlayURIs(URIs);
        } catch(e) {
            throw e;
        }
        return rc;
    },

    IsInstalled : function() {
        this.getControllerIfNeeded();
        
        try {
            var rc = this._controller.IsInstalled();
        } catch(e) {
            throw e;
        }
        return rc;
    },

	IsRunning : function() {
        this.getControllerIfNeeded();
        
        try {
            var rc = this._controller.IsRunning();
        } catch(e) {
            throw e;
        }
        return rc;
    },

    IsPlaying : function() {
        this.getControllerIfNeeded();
        
        try {
            var rc = this._controller.IsPlaying();
        } catch(e) {
            throw e;
        }
        
        return rc;
    },

    // IFoxyTunesService
    GetCurrentTrackData : function(bCached) {
        if (!bCached) {
            // really query the player for track data
            this.GetCurrentTrackTitle();
        } else {
            // return the last known track data
            // this is typically used by clients that got a notification
            // that something changed (sent by service) and want to know    
            // the most recent data, without actually quering the player
            //
            // add track length and position to track data if needed
            // this is done only for cached results, because it's assumed
            // that when using caching, both track data and its length/position
            // are updated together periodically. otherwise, the returned track
            // length and/or position could belong to another (not current) track
            if (!this._currentTrackData.length && this._currentTrackLength) {
                this._currentTrackData.length = this._currentTrackLength;
            }
            if (!this._currentTrackData.position && this._currentTrackPosition) {
                this._currentTrackData.position = this._currentTrackPosition;
            }
        }

        var unicodeTrackData = TrackDataUtils.toUnicode(this._currentTrackData);
        return unicodeTrackData;
    },

    GetPlaybackHistory : function() {
        return this._playbackHistory.getTracks();
    },

    SignatunesCheckForUpdates : function(params) {
        var paramsWrapped = params ? params.wrappedJSObject : null;
        this.foxyTunesSignatureSitesUpdateService.checkForUpdates(paramsWrapped);
    },

    // nsIObserver
    observe: function(subject, topic, data) {
        switch (topic) {
            case "foxytunes-poll-player-state-internal" :
                this.notifyPlayerStateChanged(data);
                break;
            case "nsPref:changed" : 
                this.prefChanged(data);
                break;
            case "quit-application" :
                this._observerService.removeObserver(this, "quit-application");
                this._pollTimer.cancel();
                this._pollTimer = null;
                this._prefBranch2.removeObserver("", this);
                break;
        }
    },

    // nsITimerCallback
    notify: function(aTimer) {
        if (aTimer == this._pollTimer) {
            if (!this._controller) {
                // poll timer should trigger player connection
                return;
            }            

            if (this._controller.wrappedJSObject) {
                this._controller.wrappedJSObject._bIsPolling = true;
            }

            // the following will query the player for needed 
            // data, cache everything and notify the observers if needed
            this.GetCurrentTrackTitle();

            if (this._controller.wrappedJSObject) {
                this._controller.wrappedJSObject._bIsPolling = false;
            }


            this.GetCurrentTrackLength();

            var prevPosition = this._currentTrackPosition;
            var currPosition = this.GetCurrentTrackPosition();

            this.notifyPlayerStateChanged();

            if (this._bCurrentTrackTitleIsDirty) {
                this.notifyTrackChanged('track-title-changed-internal');
                this._bCurrentTrackTitleIsDirty = false;
            }

            if (prevPosition != currPosition) {
                this.notifyTrackChanged('track-position-changed-internal');
            }

            this.switchPlayerIfNeeded();
            if (this._firstTimerSwitchBack) {
                this.switchBackFromAutoSwitchIfNeeded();
                this._firstTimerSwitchBack = false;
            }
		}
    },

	// nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.IFoxyTunesService)  ||
            iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    },

    // private
    initPlayerPropStates: function() {
        if (this._playerPropState) delete this._playerPropState;
        var me = this;
        this._playerPropState = new Object();
        this._playerPropState['props'] = new Array('mute', 'playpause', 'showhide');
        this._playerPropState['mute'] = new Object();
        this._playerPropState['mute'].updateFunc = function () {return me.IsMuted()};
        this._playerPropState['mute'].state = false;
        this._playerPropState['playpause'] = new Object();
        this._playerPropState['playpause'].updateFunc = function () {return me.IsPlaying()};
        this._playerPropState['playpause'].state = false;
        this._playerPropState['showhide'] = new Object();
        this._playerPropState['showhide'].updateFunc = function () {return me.shouldShowPlayer()};
        this._playerPropState['showhide'].state = true;
    },

    notifyPlayerStateChanged: function(bForceNotify) {
        if (typeof bForceNotify == 'undefined') bForceNotify = 'false';
        bForceNotify = (bForceNotify == 'true') ? true : false;

        for (var i=0; i<this._playerPropState['props'].length; i++) {
            var prop = this._playerPropState['props'][i];
            var prevPropState = this._playerPropState[prop].state;
            var currPropState = prevPropState;
            try {
                currPropState = this._playerPropState[prop].updateFunc();
				this._playerPropState[prop].state = currPropState;
            } catch (e) {};
            if (bForceNotify || (currPropState != prevPropState)) {
                this.notifyPropState(prop);
            }
        }
    },

    notifyPlayerState: function() {
        for (var i=0; i<this._playerPropState['props'].length; i++) {
            this.notifyPropState(this._playerPropState['props'][i]);
        }
    },

    notifyPropState: function(prop) {
        var data = {};
        data.prop = prop;
        data.state = this._playerPropState[prop].state;
        data.wrappedJSObject = data;
        this._observerService.notifyObservers(data, "foxytunes-player-state-changed-internal", "");
    },

    shouldShowPlayer: function () {
        this.getControllerIfNeeded();
        
        try {
            var rc = this._controller.IsPlayerWindowObscured();
        } catch(e) {
            // In case of player that didn't implement IsPlayerWindowObscured
            // shouldShowPlayer return true if the player isn't running
            var isRunning = this.IsRunning();
            if (!isRunning) {
                this._playerPropState['showhide'].state = true;
                return true;
            } else {
                throw e
            }
        };

        try {
            if (rc) {
                var rc = !this._controller.IsPlayerWindowAtForeground();
            }
        } catch(e) {};

        return rc;
    },

    getControllerIfNeeded : function() {
        if (this._controller != null) {
            return;
        }
        
        this._controller = Components.classes[this._prefs.playerClass].
                            getService(Components.interfaces.mozIFoxyTunes);
    },

    initAutoSwitchPlayers: function() {
        var bDisableAutoSwitching = this.getPrefWithDefault("disable_auto_switching", 'bool', false);
        if (!bDisableAutoSwitching) {
            this._autoSwitchPlayerClasses = ["@foxytunes.org/FoxyTunes/YahooMediaPlayer;1", "@foxytunes.org/FoxyTunes/YouTube;1"];
        } else {
            this._autoSwitchPlayerClasses = [];
        }
        this._autoSwitchPlayers = new Object();
        for (i in this._autoSwitchPlayerClasses) {
            var playerClass = this._autoSwitchPlayerClasses[i];
            this.addPlayerToAutoSwitchList(playerClass); 
        }   
    },

    addPlayerToAutoSwitchList: function(playerClass) {
        if (this._autoSwitchPlayers[playerClass]) return;

        this._autoSwitchPlayers[playerClass] = new Object()
        var controller = Components.classes[playerClass].
                                                        getService(Components.interfaces.mozIFoxyTunes);
        this._autoSwitchPlayers[playerClass].controller = controller;
        this._autoSwitchPlayers[playerClass].isPlaying = false;

            try {
            this._autoSwitchPlayers[playerClass].isPlaying = controller.IsPlaying();
            } catch (e) {};
    },

    switchBackFromAutoSwitchIfNeeded: function() {
        // Switch from auto-switch player (e.g. YMP) back to last player in case 
        // the auto-switch player is closed (not found in any tab)
        if (this._autoSwitchPlayerClasses.toString().indexOf(this._prefs.playerClass) == -1) return false;

        try {
        if (!this._controller.IsRunning()) { 
                var recentPlayers = this.getPrefWithDefault('recent_players', 'char', '');
            var recentPlayerClass = recentPlayers.split(',');
            for (var i=0; i<recentPlayerClass.length; i++) {
                var playerClass = recentPlayerClass[i];
                if (playerClass == "") continue;
                var isRunning = false;
                try {
                    isRunning = Components.classes[playerClass].getService(Components.interfaces.mozIFoxyTunes).IsRunning();
                } catch (e) {};
                if (isRunning) {
                    this._observerService.notifyObservers(null, "foxytunes-internal-switch-player", playerClass);
                    return true;
        }
            }
        }
        } catch (e) {};

        return false;
    },

    switchPlayerIfNeeded: function() {
        if (this._allowSwitchBack && this._autoSwitchPlayers[this._prefs.playerClass]) {
            if (this.switchBackFromAutoSwitchIfNeeded()) return;
        } 

        for (var playerClass in this._autoSwitchPlayers) {
        var bNeedSwitch = false;
            var player = this._autoSwitchPlayers[playerClass];
            var controller = this._autoSwitchPlayers[playerClass].controller;
                
            try {
                var isPlaying = controller.IsPlaying();
                    if (isPlaying && !this._autoSwitchPlayers[playerClass].isPlaying) {
                bNeedSwitch = true;
            }
                    this._autoSwitchPlayers[playerClass].isPlaying = isPlaying;
            } catch (e) {}; 

            // Don't switch if its already the current player
            if (this._controller == player.controller) continue;

            if (bNeedSwitch) {
                this._observerService.notifyObservers(null, "foxytunes-internal-switch-player", playerClass);
                this._allowSwitchBack = true;
            } 
        }
    },

    notifyTrackChanged : function(data) {
        this._observerService.notifyObservers(null, "foxytunes-internal-track-data", data);
    },

    getPrefWithDefault : function(prefName, prefType, defaultValue) {
        var value = defaultValue;
        var getterFunc = null;
        switch (prefType) {
            case 'bool':
                getterFunc = this._prefBranch.getBoolPref;
                break;
            case 'int':
                getterFunc = this._prefBranch.getIntPref;
                break;
            case 'char':
                getterFunc = this._prefBranch.getCharPref;
                break;
        }

        if (getterFunc) {
            try {
                value = getterFunc(prefName);
            } catch(e) {}
        }
        return value;
    },

    readPrefs : function() {
        this._prefs.trackDataPollInterval = this.getPrefWithDefault('title_poll_interval', 'int', 5000);
        this._prefs.bURLUnescape = this.getPrefWithDefault('url_unescape', 'bool', false);
        this._prefs.characterEncoding = this.getPrefWithDefault('character_encoding', 'char', 'default');

        // TBD: Linux/Mac will probably want another default :)
        this._prefs.playerClass =  
                this.getPrefWithDefault('player_class', 'char', '@foxytunes.org/FoxyTunes/WindowsMediaPlayer;1');

    },

    prefChanged : function(prefName) {
        switch (prefName) {
            case 'player_class':
                // Do not try to switch back on first time if user changed player manually
                this._firstTimerSwitchBack = false; 

                // If switching from auto-switch player than init auto switch players
                // in order to update isPlaying
                for (var playerClass in this._autoSwitchPlayers) {
                    var player = this._autoSwitchPlayers[playerClass];
                
                    if (this._controller != player.controller) continue;
            
                    this.initAutoSwitchPlayers();
                    this._allowSwitchBack = false;
                    break;
                }

                // invalidate the controller object. it will be re-initialized on next operation
                this._controller = null;
                break;
            case 'disable_auto_switching':
                this.initAutoSwitchPlayers();
                break;
        }
        this.readPrefs();

        if (prefName == 'player_class') {
            this.initPlayerPropStates();
            this.notifyPlayerState();
            this.notifyTrackChanged('track-title-changed-internal');
        }
    },

    charsetStringToUTF8 : function(string, srcCharset) {
           	this._unicodeConverter.charset = srcCharset;
        	var unicodeStr = this._unicodeConverter.ConvertToUnicode(string);
            this._unicodeConverter.charset = 'UTF-8';
            var utf8Str = this._unicodeConverter.ConvertFromUnicode(unicodeStr);
            utf8Str += this._unicodeConverter.Finish();
            return utf8Str;
    },

    getTrackDataItemFromXML : function(trackDOM, item) {
        var value = null;
        var nodeList = trackDOM.getElementsByTagName(item)[0];
        if (nodeList) {
            var value = nodeList.firstChild.nodeValue;
        }
        return value;
    },

    getTrackDataFromXML : function(trackXML) {
    	var trackDOM = this._DOMParser.parseFromString(trackXML, "text/xml");	

        this._currentTrackData.aatWasGuessed = false;
        this._currentTrackData.originalTitle = this.getTrackDataItemFromXML(trackDOM, 'display-name');
        this._currentTrackData.artist = this.getTrackDataItemFromXML(trackDOM, 'artist');
        this._currentTrackData.album = this.getTrackDataItemFromXML(trackDOM, 'album');
        this._currentTrackData.title = this.getTrackDataItemFromXML(trackDOM, 'title');
        this._currentTrackData.location   = this.getTrackDataItemFromXML(trackDOM, 'uri');
        this._currentTrackData.albumCoverURL   = this.getTrackDataItemFromXML(trackDOM, 'album-cover-uri');
        this._currentTrackData.isVideo   = this.getTrackDataItemFromXML(trackDOM, 'is-video');
        this._currentTrackData.isVideo = this._currentTrackData.isVideo ? this._currentTrackData.isVideo : 'unknown';

        if (!this._currentTrackData.album && !this._currentTrackData.artist) {
            this._currentTrackData.aatWasGuessed = true;
        }
        
        var imageUrl = this._currentTrackData.albumCoverURL;
        if (imageUrl) {
            if (!imageUrl.match(/^file:\/\//i) && !imageUrl.match(/^http:\/\//i)) {
                // ilegal image url
                this._currentTrackData.albumCoverURL = null;
            }
            if (imageUrl.match(/.*\/nopicture[^\/]*$/i) ) {
                // yahoo dummy image 
                this._currentTrackData.albumCoverURL = null;
            }
            if (!this.isImageUrl(imageUrl) ) {
                // In rhapsody and YMP we do not have the image extension - it redirecting regular url
                if (!this._prefs.playerClass.match(/rhapsody/i) &&
                    !this._prefs.playerClass.match(/yahoomediaplayer/i)) {
                    this._currentTrackData.albumCoverURL = null;
                }
            }
        }
    },

    isImageUrl : function(imageUrl) {
        if (!imageUrl) {
            return false;
        }
        if (!imageUrl.match(/\.jpg$/i) 
                && !imageUrl.match(/\.jpeg$/i) 
                && !imageUrl.match(/\.gif$/i) 
                && !imageUrl.match(/\.png$/i) ) {

            return false;
        }
        return true;
    },

    normalizeTitle : function(str) {
        if (!str) {
            return null;
        }
        str = str.replace(/"/g, '');
        str = str.replace(/\([^\)]*\)/g, '');
        str = str.replace(/\[[^\]]*\]/g, '');
        str = str.replace(/[()]/g, '');
        str = str.replace(/[\[\])]/g, '');
        str = str.replace(/\.mp3$/, '');
        return str;
    },

    IsTitleEmpty : function(title) {
        if (!title) return true;

        if (title.substr(0, 7) == '<track>') {
            var trackDOM = this._DOMParser.parseFromString(title, "text/xml");	
            title = this.getTrackDataItemFromXML(trackDOM, 'title');
            if (!title) title = this.getTrackDataItemFromXML(trackDOM, 'display-name');
        }

        if (!title || title == "") return true;

        return false;
    },

    fixCurrentTrackData : function() {
        var displayName = this._currentTrackData.originalTitle;
        if (!displayName) {
            return;
        }
        var normalizedDisplayName = this.normalizeTitle(displayName);
        this._currentTrackData.normalizedOriginalTitle = normalizedDisplayName;

        var bITunesRadio = this._prefs.playerClass.match(/itunes/i) &&
                           !this._currentTrackData.artist &&
                           !this._currentTrackData.album && 
                            this._currentTrackData.title;
        var bArtistTitleMissing = !this._currentTrackData.artist &&
                                  !this._currentTrackData.title; 
        if (!this._currentTrackData.aatWasGuessed) {
            if (bArtistTitleMissing || bITunesRadio) {
                if (normalizedDisplayName.match(/(.*) - (.*)/)) {
                    this._currentTrackData.artist = RegExp.$1;
                    this._currentTrackData.title  = RegExp.$2;
                }
            }
        } else {
            if (this._prefs.playerClass.match(/foobar/i) && displayName.match(/\[([^#]+)#.*\]/)) {
                this._currentTrackData.album  = RegExp.$1;
            }

            if (this._prefs.playerClass.match(/winamp3/i) && normalizedDisplayName.match(/^\d+\. (.*)/)) {
                normalizedDisplayName  = RegExp.$1;
            }

            if (this._prefs.playerClass.match(/noatun/i) && displayName.match(/^\[([^\]]*)\] - /)) {
                // displayName= [artist] - title (bit rate)
                // normalizedDisplayName= - title
                // res = artist - title
                normalizedDisplayName  = RegExp.$1 + normalizedDisplayName;
            }
            

            if ((this._prefs.playerClass.match(/realplayer/i) || this._prefs.playerClass.match(/musikcube/i)) &&
                        normalizedDisplayName.match(/(.*) - (.*)/)) {
                this._currentTrackData.artist = RegExp.$2;
                this._currentTrackData.title  = RegExp.$1;
            } else if (normalizedDisplayName.match(/(.*) by (.*)/)) {
                this._currentTrackData.artist = RegExp.$2;
                this._currentTrackData.title  = RegExp.$1;
            } else if (normalizedDisplayName.match(/(.*) - (.*)/)) {
                this._currentTrackData.artist = RegExp.$1;
                this._currentTrackData.title  = RegExp.$2;
            } else if (normalizedDisplayName.match(/(.*) \/ (.*)/)) {
                this._currentTrackData.artist = RegExp.$1;
                this._currentTrackData.title  = RegExp.$2;
            } else if (normalizedDisplayName.match(new RegExp("(.*) " + String.fromCharCode(0xE2, 0x80, 0x93) + " (.*)"))) {
                // Spliting unicode hypen - \u2013
                this._currentTrackData.artist = RegExp.$1;
                this._currentTrackData.title  = RegExp.$2;
            }
        }

        if (!this._currentTrackData.title) {
            this._currentTrackData.title = normalizedDisplayName;
        }

        TrackDataUtils.normalize(this._currentTrackData);
    },

    playiTunesM3UIfNeeded : function(URIs) {
        var uriArray = URIs.split('\t');
        if ((uriArray.length == 1) && 
            this._prefs.playerClass.match(/itunes/i) && 
            URIs.match(/^http.*\.m3u$/i)) {
            var url = URIs;
            var xmlHTTPrequest = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].
                                    createInstance(Components.interfaces.nsIXMLHttpRequest);
        	xmlHTTPrequest.open('get', url, true);
            var me = this;
            xmlHTTPrequest.onload = function() {
                 var contents = xmlHTTPrequest.responseText;
                 var lines = contents.split(/[\n\r]/);
                 var urls = [];
                 for (var i=0; i < lines.length; i++) {
                    if (!lines[i].match(/^#/)) {
                        urls.push(lines[i]);
                    }
                 }
                 me.PlayURIs(urls.join('\t'));
            }
            xmlHTTPrequest.send(null);
            // we handled the play request
            return true;
        }

        // we didn't handle
        return false;
    }
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesService]);
} catch (e) {};    
