/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/
var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);

if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes-components-js/content/players/FoxyTunesJSPlayerBase.js");
}

var playerConfig = new FoxyTunesJSPlayerConfig();
playerConfig.bUseGlobalVolumeController = false;
playerConfig.bUseChromeToContentBridge = false;

playerConfig.commandStrMappings["Play"] = "play";
playerConfig.commandStrMappings["Pause"] = "pause";
playerConfig.commandStrMappings["Stop"] = "stop";
playerConfig.commandStrMappings["Next"] = "next";
playerConfig.commandStrMappings["Previous"] = "previous";
playerConfig.commandStrMappings["ToggleMute"] = "toggleMute";
playerConfig.commandStrMappings["SetVolume"] = "setVolume:%value%";
playerConfig.commandStrMappings["GetVolume"] = "getVolume";
playerConfig.commandStrMappings["GetCurrentTrackLength"] = "getTrackLength";
playerConfig.commandStrMappings["GetCurrentTrackPosition"] = "getTrackPosition";
playerConfig.commandStrMappings["GetCurrentTrackTitle"] = "getTrackTitle";
playerConfig.commandStrMappings["SetCurrentTrackPosition"] = "setTrackPosition:%value%";
playerConfig.commandStrMappings["ShowPlayer"] = "showPlayer";
playerConfig.commandStrMappings["HidePlayer"] = "hidePlayer";

function FoxyTunesSongbirdController() {
    this.init(playerConfig);

    this.portNumber = 3008;
    this.platform = this.getPlatform();

    this.cmdPaddingLeft = "FT_START>>";
    this.cmdPaddingRight = "<<FT_END";

    this.transportService = Components.classes["@mozilla.org/network/socket-transport-service;1"]
                                .getService(Components.interfaces.nsISocketTransportService);
    this.initService();
}

FoxyTunesSongbirdController.prototype = new FoxyTunesJSPlayerBase();
FoxyTunesSongbirdController.prototype.classID = Components.ID("{C6D5E58A-760C-11DD-8CC6-280B56D89593}");

FoxyTunesSongbirdController.prototype.processCmdInput["SetVolume"] = function(value) { return value/100; };
FoxyTunesSongbirdController.prototype.processCmdInput["SetCurrentTrackPosition"] = function(value) { return value*1000; };

FoxyTunesSongbirdController.prototype.defaultCmdHandler = function(player, cmd, value) {
    this.sendCommand(cmd);
    switch (cmd) {
        case this.playerConfig.commandStrMappings["GetVolume"]: 
            return this._lastVolume;
        case this.playerConfig.commandStrMappings["GetCurrentTrackTitle"]:
            return this._lastTitle;
        case this.playerConfig.commandStrMappings["GetCurrentTrackLength"]:
            return this._lastLength;
        case this.playerConfig.commandStrMappings["GetCurrentTrackPosition"]:
            return this._lastPosition;
    }
};

FoxyTunesSongbirdController.prototype.findPlayer = function() { return true; };
FoxyTunesSongbirdController.prototype.IsMuted = function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; };
FoxyTunesSongbirdController.prototype.IsRunning = function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; };
FoxyTunesSongbirdController.prototype.IsPlaying = function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; };
FoxyTunesSongbirdController.prototype.IsPlayerWindowObscured = function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; };

FoxyTunesSongbirdController.prototype.IncreaseVolume = function() {
    return this.SetVolume(this._lastVolume+10);
};

FoxyTunesSongbirdController.prototype.DecreaseVolume = function() {
    return this.SetVolume(this._lastVolume-10);
};

FoxyTunesSongbirdController.prototype.SetOptions = function(options) {
    var optionsArr = options.split(' ');
    for (var i=0; i < optionsArr.length; i++) {
        if (optionsArr[i] == '-port') {
            try {
                this.portNumber = optionsArr[i+1];
                this.updatePortCfgFile();
            } catch(e) {
                dump('Error: couldn\'t get songbird port from command line\n');
            }
        }
    }
    return true;
};

FoxyTunesSongbirdController.prototype.IsInstalled = function() {
    var isInstalled = false;
    switch (this.platform) {
        case "windows": isInstalled = this.isInstalledWindows();
                        break;
        case "mac" : isInstalled = this.isInstalledMac();
		             break;
        default: throw Components.results.NS_ERROR_NOT_IMPLEMENTED
                 break;
    }

    return isInstalled;
};

FoxyTunesSongbirdController.prototype.initService = function() {
    this._lastVolume = 50;
    this._lastPosition = 0;
    this._lastLength = 0;
    this._lastTitle = '';

    if (this.outStream) this.outStream.close();
    if (this.inputStream) this.inputStream.close();
    if (this.transport) this.transport.close(0);

    this.outStream = null;
    this.inputStream = null;
    this.transport = null;
};

FoxyTunesSongbirdController.prototype.getPlatform = function() {
    var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                        .getService(Components.interfaces.nsIXULRuntime);
    var platform = appInfo.OS.toLowerCase();

    if (platform.indexOf('linux') != -1) {
	    return 'linux';
	}

	if (platform.indexOf('darwin') != -1) {
	    return 'mac';
	}

	return 'windows';
};

FoxyTunesSongbirdController.prototype.isInstalledWindows = function() {
    var uninstallString = "";
    try {
        var wrk = Components.classes["@mozilla.org/windows-registry-key;1"]
                    .createInstance(Components.interfaces.nsIWindowsRegKey);
        wrk.open(wrk.ROOT_KEY_LOCAL_MACHINE,
                 "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall",
                 wrk.ACCESS_READ);
        for (var i=0; i<wrk.childCount; i++) {
            var name  = wrk.getChildName(i);
            if (name.toLowerCase().indexOf('songbird') > -1) {
                var sbKey = wrk.openChild(name, wrk.ACCESS_READ);
                uninstallString = sbKey.readStringValue("UninstallString");
            }
         }
    } catch (e) {};
    if (uninstallString == '') return false;
    uninstallString = uninstallString.substr(1, uninstallString.length-2);
    try {
        var file = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
        file.initWithPath(uninstallString);
    } catch(e) {
        return false;
    }
    return file.exists();
};
 
FoxyTunesSongbirdController.prototype.isInstalledMac = function() {
    var desktop = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("Desk", Components.interfaces.nsIFile);

    var searchFolders = new Array();
	searchFolders[0] = '/Applications';
	searchFolders[1] = desktop.path;

	var file = Components.classes["@mozilla.org/file/local;1"]
                     .createInstance(Components.interfaces.nsILocalFile);
	for (var i in searchFolders) {
        var folder = searchFolders[i];
	    try {
            file.initWithPath(folder);
	  	    if (this.isSongbirdFoundUnderPath(file)) return true;
        } catch (e) {};
	}

	return false;
};

FoxyTunesSongbirdController.prototype.isSongbirdFoundUnderPath = function(baseFolder) {
    var entries = baseFolder.directoryEntries;
	while(entries.hasMoreElements()) {
  	    var entry = entries.getNext();
	    entry.QueryInterface(Components.interfaces.nsIFile);
	    if (!entry.isDirectory()) continue;
	    if (entry.path.toLowerCase().indexOf('songbird') > -1) return true;
	}
    return false;
};

FoxyTunesSongbirdController.prototype.updatePortCfgFile = function() {
    var cfgFile = Components.classes["@mozilla.org/file/directory_service;1"]
                     .getService(Components.interfaces.nsIProperties)
                     .get("TmpD", Components.interfaces.nsIFile);
        
    cfgFile.append("FoxyTunesSupportForSongbird");
    cfgFile.append("port.cfg");
    if (cfgFile.exists()) cfgFile.remove(false);
    cfgFile.create(Components.interfaces.nsILocalFile.NORMAL_FILE_TYPE, 0644);
    var outputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
                            createInstance( Components.interfaces.nsIFileOutputStream );
    outputStream.init(cfgFile, 0x04 | 0x08 | 0x20,  0644, 0);
    var text = this.portNumber;
    outputStream.write(text, text.length);
	outputStream.close();
};

FoxyTunesSongbirdController.prototype.handleSongbirdResponce = function(resp) {
    var respArr = resp.split(':');
    if (respArr.length < 2) return;
    var cmd = respArr.shift();
    var value = respArr.join(':');
    switch (cmd) {
        case 'getVolume': 
            this._lastVolume = Math.round(value*100); 
            break;
        case 'getTrackLength':  
            this._lastLength = Math.round(value/1000);
            break;
        case 'getTrackPosition': 
            this._lastPosition = Math.round(value/1000);
            break;
        case 'getTrackTitle':
            if (value != '') this._lastTitle = value;
            break;
    }
};

FoxyTunesSongbirdController.prototype.initTransport = function() {
    this.transport = this.transportService.createTransport(null, 0, '127.0.0.1', this.portNumber, null);
    this.outStream = this.transport.openOutputStream(0, 0, 0);
    this.inputStream = this.transport.openInputStream(0, 0, 0);

    var inStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
                    .createInstance(Components.interfaces.nsIScriptableInputStream);
    inStream.init(this.inputStream);

    var me = this;

    var dataListener = {
        data : "",
        onStartRequest: function(request, context) { },
        onStopRequest: function(request, context, status) {
            inStream.close();
            me.initService();
        },
        onDataAvailable: function(request, context, inputStream, offset, count) {
            try {
                this.data += inStream.read(count);
                var cmdStartIndex = 0
                var cmdEndIndex = 0
                do {
                    cmdStartIndex = this.data.indexOf(me.cmdPaddingLeft);
                    cmdEndIndex = this.data.indexOf(me.cmdPaddingRight);
                    if ((cmdStartIndex >= 0) && (cmdEndIndex > 0)) {
                        me.handleSongbirdResponce(this.data.slice(cmdStartIndex+me.cmdPaddingLeft.length, cmdEndIndex));
                        this.data = this.data.substr(cmdEndIndex+me.cmdPaddingRight.length);
                    }
                } while ((cmdStartIndex >= 0) && (cmdEndIndex > 0))
            } catch (e) {
                dump("ERROR on onDataAvailable " + e + ", count: " + count + "\n");
            } 
        },
    };

    var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"].
                    createInstance(Components.interfaces.nsIInputStreamPump);
    pump.init(this.inputStream, -1, -1, 0, 0, false);
    pump.asyncRead(dataListener, null);
};

FoxyTunesSongbirdController.prototype.sendCommand = function(cmd) {
    if (!this.transport) {
        this.initTransport(); 
    }

    cmd = this.cmdPaddingLeft + cmd + this.cmdPaddingRight;
    this.outStream.write(cmd, cmd.length);
    this.outStream.flush();
};

try {
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesSongbirdController]);
} catch(e) {};
