/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/

/********* FoxyTunesXBMCController *********/

function FoxyTunesXBMCController() {
    this._ip = null;

    this._lastVolume = 50;
    this._lastPlaying = {};

    this._firstTime = true;
    this._lastTitle = '';
    this._lastPosition = 0;
    this._lastLength = 0;

    var date = new Date();
    this._lastCommandTime = date.getTime();

    this._windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Components.interfaces.nsIWindowMediator);

    var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);
    if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes/content/utils/trackDataBuilder.js");
    }
}

FoxyTunesXBMCController.prototype = {
    classID : Components.ID("{35125915-C29D-48f6-85F3-38161D4A781E}"),

    Play : function() {
        //this.sendCommand(this.getPlayerControlCommand('Play'));
        this.sendCommand('play');
    },

    Pause : function() {
        //this.sendCommand(this.getPlayerControlCommand('Play'));
        this.sendCommand('pause');
    },

    ForcePause : function() {
        this.sendCommand('pause');
    },

    Stop : function() {
        this.sendCommand('stop');
    },

    Next : function() {
        //this.sendCommand(this.getPlayerControlCommand('Next'));
        this.sendCommand('playNext');
    },

    Previous : function() {
        //this.sendCommand(this.getPlayerControlCommand('Previous'));
        this.sendCommand('playPrev');
    },

    ToggleMute : function() {
        this.sendCommand('ExecBuiltIn&parameter=Mute()');
    },

    SetVolume : function(value) {
        this.sendCommand('setvolume&parameter=' + value);
        this._lastVolume = value;
    },

    GetVolume : function() {
        var me = this;
        this.sendCommand('getvolume', function(value) { 
            me.printTrace('----> volume received: -' + value + '-');
            value = me.removeHTML(value, true);
            me.printTrace('----> volume replaced: -' + value + '-');
            if (me.isNumber(value)) {
                me._lastVolume = value; 
            }
        } );
        return this._lastVolume;
    },

    IncreaseVolume : function() {
        var me = this;
        function increaseVolumeCallback(value) {
            me._lastVolume = value; 
            me._lastVolume += 10;
            if (me._lastVolume > 100) me._lastVolume = 100;
            me.SetVolume(me._lastVolume);
        }

        this.sendCommand('getvolume', increaseVolumeCallback);
    },

    DecreaseVolume : function() {
        var me = this;
        function decreaseVolumeCallback(value) {
            me._lastVolume = value; 
            me._lastVolume -= 10;
            if (me._lastVolume < 0) me._lastVolume = 0;
            me.SetVolume(me._lastVolume);
        }

        this.sendCommand('getvolume', decreaseVolumeCallback);
    },

    GetCurrentTrackTitle : function() {

        var date = new Date();
        var now = date.getTime();

        if (((now - this._lastCommandTime) < 1000) && !this._firstTime) {
             return this._lastTitle;
        }

        this._firstTime = false;

        var date = new Date();
        var now = date.getTime();
        this._lastCommandTime = now;

        var me = this;
        function parseCurrentlyPlaying(value) {
            value = me.removeHTML(value, false);

            //dump(value + '\n\n\n');

            // <li>Filename:bla.mp3
            // <li>SongNo:2
            // <li>Type:Audio
            // <li>Title:MySongTitle
            // <li>Artist:MyArtist
            // <li>Album:MyAlbum
            // <li>Genre:Rap/R&B
            // <li>Bitrate:128
            // <li>Samplerate:44
            // <li>Time:0:48
            // <li>Duration:3:28
            // <li>Percentage:23

            var lines = value.split('<li>');
            me._lastPlaying = {};
            for (var i=0; i < lines.length; i++) {
                var line = lines[i];
                if (line.match(/([^:]+):(.*)/)) {
    				var name = RegExp.$1;
	    			var value = RegExp.$2;
                    me._lastPlaying[name] = value;
                }
            }

            me._lastLength = 0;
            if (typeof me._lastPlaying['Duration'] != 'undefined') {
                me._lastLength = me.timeStrToSeconds(me._lastPlaying['Duration']);
            } 
            me._lastPosition = 0;
            if (typeof me._lastPlaying['Time'] != 'undefined') {
                me._lastPosition = me.timeStrToSeconds(me._lastPlaying['Time']);
            }

            var dataBuilder = new TrackDataBuilder();

            var track = '';
            var album = '';
            var artist = '';
            if (typeof me._lastPlaying['Title'] != 'undefined') {
                track = me._lastPlaying['Title'];
                if (track) {
                    dataBuilder.addTitle(track);
                }
            }
            if (typeof me._lastPlaying['Artist'] != 'undefined') {
                artist = me._lastPlaying['Artist'];
                if (artist) {
                    dataBuilder.addArtist(artist);
                }
            }
            if (typeof me._lastPlaying['Album'] != 'undefined') {
                album = me._lastPlaying['Album'];
                if (album) {
                    dataBuilder.addAlbum(album);
                }
            }

            var displayName = '';
            if (artist) { 
                displayName = artist;
                if (track) {
                   displayName += ' - ' + track;
                }
            } else if (track) {
                displayName = track;
            }
            dataBuilder.addDisplayName(displayName);
            me._lastTitle = dataBuilder.getData();

            //me.printTrace('track: ' + track);
            //me.printTrace('artist: ' + artist);
            //me.printTrace('album: ' + album);
        }

        this.sendCommand('getcurrentlyplaying', parseCurrentlyPlaying);

        return this._lastTitle;
    },

    GetCurrentTrackLength : function() {
        this.GetCurrentTrackTitle();
        return this._lastLength;
    },

    GetCurrentTrackPosition : function() {
        this.GetCurrentTrackTitle();
        return this._lastPosition;
    },

    ShowPlayer : function() {
        this.openXBMC();
        return false;
    },

    HidePlayer : function() {
        return false;
    },

    PlayFile : function() {
        var promptService = Components.classes['@mozilla.org/embedcomp/prompt-service;1']
                                    .getService(Components.interfaces.nsIPromptService);


        var input = {value: ""};
        var check = {value: null};
        var bOk = promptService.prompt(null, '', this.getLocaleString('openURL'), input, null, check);
        if (bOk) {
            var urls = input.value;
            urls = urls.replace(/^ */, '');
            urls = urls.replace(/ *$/, '');
            urls = urls.replace(/ /g, '\t');
            this.PlayURIs(urls);
        }
        return true;
    },

    SetOptions  : function(options) {
        var optionsArr = options.split(' ');
        for (var i=0; i < optionsArr.length; i++) {
            if (optionsArr[i] == '-ip') {
                try {
                    this._ip = optionsArr[i+1];
                } catch(e) {
                    dump('Error: couldn\'t get XBMC IP from command line\n');
                }
            }
        }
        return true;
    },

    SetCurrentTrackPosition : function(pos) {
       if (!this._lastLength) return;

       var percentage = Math.round((pos / this._lastLength) * 100);
       this.sendCommand('seekpercentage&parameter=' + percentage);
       this._lastPosition = pos;
    },

    IsMuted : function() {
       throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    PlayURIs : function(URIs) {
        var URIArr = URIs.split('\t');
        var me = this;
        if (URIArr.length == 1) {
            this.sendCommand('playfile&parameter=' + URIs);
        } else {
            var commands = [];
            commands.push('setcurrentplaylist&parameter=0');
            commands.push('clearplaylist');
            for (var i=0; i < URIArr.length; i++) {
                var uri = URIArr[i];
                commands.push('addtoplaylist&parameter=' + uri);
            }
            commands.push('setplaylistsong&parameter=0');
            function sendNextCommand() {
                if (commands.length == 0) return;
                var cmd = commands.shift();
                me.sendCommand(cmd, sendNextCommand);
            }           
            sendNextCommand();
        }
    },

    IsInstalled : function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsRunning : function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsPlaying : function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsPlayerWindowObscured: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

    IsPlayerWindowAtForeground: function() {
        throw Components.results.NS_ERROR_NOT_IMPLEMENTED;
    },

	// nsISupports
    QueryInterface: function (iid) {
        if (iid.equals(Components.interfaces.mozIFoxyTunes) ||
			iid.equals(Components.interfaces.nsISupports)) {
               return this;
		}

	   throw Components.results.NS_ERROR_NO_INTERFACE;       
    },

    // private:
    timeStrToSeconds : function(str) {
        var timeArr = str.split(':');
        if (timeArr.length == 1) return str * 1;

        if (timeArr.length == 2) return timeArr[0]*60 + timeArr[1]*1;

        if (timeArr.length == 3) return timeArr[0]*60*60 + timeArr[1]*60 + timeArr[2]*1;
    },

    openXBMC : function() {
            if (!this._ip) return null;

            const XBMCHome = 'http://' + this._ip + '/';
            var win = this._windowMediator.getMostRecentWindow('navigator:browser');
            if (win) {
           		var newTab = win.getBrowser().addTab(XBMCHome);
                win.getBrowser().selectedTab = newTab;
            } else {
                var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
                                    .getService(Components.interfaces.nsIWindowWatcher);
                var array = Components.classes["@mozilla.org/supports-array;1"]
                                .createInstance(Components.interfaces.nsISupportsArray);
                var url = Components.classes["@mozilla.org/supports-string;1"]
                                .createInstance(Components.interfaces.nsISupportsString);
                url.data = XBMCHome;

                array.AppendElement(url);
                windowWatcher.openWindow(windowWatcher.activeWindow,
                                 "chrome://browser/content/browser.xul",
                                 "_blank",
                                 "chrome,all,dialog=no",
                                 array);
           }
    },

    getPlayerControlCommand : function(command) {
        return 'ExecBuiltIn&parameter=PlayerControl(' + command + ')';
    },

    getCommandURL : function(cmd) {
        if (!this._ip) return null;

        var url = 'http://' + this._ip + '/xbmcCmds/xbmcHttp?command=' + cmd;
        return url;
    },


    sendCommand : function(cmd, callback) {
        var url = this.getCommandURL(cmd);
        if (!url) return false;

        //this.printTrace(url);
        var xmlHTTPrequest = Components.classes['@mozilla.org/xmlextras/xmlhttprequest;1'].
                                    createInstance(Components.interfaces.nsIXMLHttpRequest);
        xmlHTTPrequest.open('get', url, true);
        var me = this;
        xmlHTTPrequest.onreadystatechange = function() {
                if (xmlHTTPrequest.readyState != 4) return;
                if (xmlHTTPrequest.status != 200) return;
                var contents = xmlHTTPrequest.responseText;
                if (callback) {
                    callback(contents);
                }
        }
        xmlHTTPrequest.send(null);
    },

    printTrace : function(str) {
        return; // trace disabled in production
        dump(str + '\n');
        var console = Components.classes['@mozilla.org/consoleservice;1'].
                        getService(Components.interfaces.nsIConsoleService);
        console.logStringMessage(str);
    },

    getLocaleString : function(strName) {
       var stringBundleService = Components.classes['@mozilla.org/intl/stringbundle;1'].
                                    createInstance(Components.interfaces.nsIStringBundleService);
       var bundle = stringBundleService.createBundle('chrome://foxytunes/locale/foxytunes.properties');
       var val = '';
       try { 
           val = bundle.GetStringFromName(strName);
       } catch(e) {
            var bundle = stringBundleService.createBundle('chrome://foxytunes/content/locale/en-US/foxytunes.properties');
            try {
                val = bundle.GetStringFromName(strName);
            } catch(e) {}
       }

       return val;
    },

    removeHTML : function(str, bRemoveLIElements) {
        var retStr = str;
        retStr = retStr.replace(/[\r\n]/g, '');
        retStr = retStr.replace(/<html>/gi, '');
        retStr = retStr.replace(/<\/html>/gi, '');

        if (bRemoveLIElements) {
            retStr = retStr.replace(/<li>/gi, '');
        }

        return retStr;
    },

    isNumber : function(x) {
    	return parseInt(x, 10) == x;
    }
}

try {
    Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesXBMCController]);
} catch(e) {};
