/*
  Copyright (c) 2009 Yahoo! Inc. 
  All Rights Reserved
*/
var scriptLoader =  Components.classes["@mozilla.org/moz/jssubscript-loader;1"].
                     getService(Components.interfaces.mozIJSSubScriptLoader);

if (scriptLoader) {
       scriptLoader.loadSubScript("chrome://foxytunes-components-js/content/players/FoxyTunesJSPlayerBase.js");
}

var playerConfig = new FoxyTunesJSPlayerConfig();
playerConfig.playerURLRegExp = new RegExp("youtube.com");
playerConfig.playerHomepage = "http://www.youtube.com";

playerConfig.commandStrMappings["Play"] = "playVideo();";
playerConfig.commandStrMappings["Pause"] = "pauseVideo();";
playerConfig.commandStrMappings["SetVolume"] = "setVolume(%value%);";
playerConfig.commandStrMappings["GetVolume"] = "getVolume();";
playerConfig.commandStrMappings["GetCurrentTrackLength"] = "getDuration();";
playerConfig.commandStrMappings["GetCurrentTrackPosition"] = "getCurrentTime();";
playerConfig.commandStrMappings["SetCurrentTrackPosition"] = "seekTo(%value%, true);";
playerConfig.commandStrMappings["IsMuted"] = "isMuted();";
playerConfig.commandStrMappings["Next"] = "yt.www.watch.queue.playNext();";

function FoxyTunesYouTubeController() {
    this.init(playerConfig);
}

FoxyTunesYouTubeController.prototype = new FoxyTunesJSPlayerBase();
FoxyTunesYouTubeController.prototype.classID = Components.ID("{CFB58D38-1597-11DE-A654-78EF55D89593}");

FoxyTunesYouTubeController.prototype.processCmdOutput["IsMuted"] = function(value) { return (value == "true") };

FoxyTunesYouTubeController.prototype.defaultCmdHandler = function(player, cmd, value) {
    if (!player) return false;
    var youtubeCmd = "FTYouTubePlayer." + cmd;
    return this.chromeToContentBridge.fireCommand(player.doc, youtubeCmd);
};

FoxyTunesYouTubeController.prototype.Previous = function() { throw Components.results.NS_ERROR_NOT_IMPLEMENTED; };

FoxyTunesYouTubeController.prototype.Stop = function() {
    return this.Pause();
}

FoxyTunesYouTubeController.prototype.IsRunning = function() {
    //Assume running if youtube tab is opened to avoid switch-back when browsing non-player pages in youtube
    return this.isYouTubeOpened();
}

FoxyTunesYouTubeController.prototype.commandHandlers["Next"] = function(player, cmd, value) {
    if (!player) return false;
    return this.chromeToContentBridge.fireCommand(player.doc, cmd);
};

FoxyTunesYouTubeController.prototype.commandHandlers["ToggleMute"] = function(player, cmd, value) {
    if (!player) return false;
    var muteCmd = this.IsMuted() ? "unMute()" : "mute()";
    return this.chromeToContentBridge.fireCommand(player.doc, "FTYouTubePlayer." + muteCmd);
};

FoxyTunesYouTubeController.prototype.commandHandlers["GetCurrentTrackTitle"] = function(player, cmd, value) {
    if (!player) return this.getTrackData();

    var title = player.doc.title;
    title = title.replace(/^YouTube *- */ , '');
   
    return this.getTrackData({"title": title, "isVideo": "yes"});
};

FoxyTunesYouTubeController.prototype.playerFoundInDoc = function(doc) {
    if (!doc.location || !doc.location.href) return false;
    if (!doc.location.href.match(this.playerConfig['playerURLRegExp'])) return false;
    return (doc.getElementById('movie_player') != null);
};

FoxyTunesYouTubeController.prototype.initPlayerDoc = function(doc) {
    this.chromeToContentBridge.init(doc);
    this.chromeToContentBridge.fireCommand(doc, 'FTYouTubePlayer = document.getElementById("movie_player")');
};

FoxyTunesYouTubeController.prototype.playerIsPlayingInDoc = function(doc) {
    return (this.chromeToContentBridge.fireCommand(doc, "FTYouTubePlayer.getPlayerState()") == "1");
};

FoxyTunesYouTubeController.prototype.isYouTubeOpened = function() {
    var typeList = ['navigator:browser', 'foxytunes:minibrowser'];
    for (var i in typeList) {
        var windows = this.services["WindowMediator"].getEnumerator(typeList[i]);
        while (windows.hasMoreElements()) {
            var wantedWindow = windows.getNext().QueryInterface(Components.interfaces.nsIDOMWindowInternal);
            // we do not want popup windows (?)
            if (!wantedWindow.toolbar.visible) continue;

            var tabbrowser = wantedWindow.getBrowser();
            var tabs = tabbrowser.mTabContainer.childNodes;
            for (var iTab=0; iTab < tabs.length; iTab++) {
                var tab = tabs[iTab];
                var browser = tabbrowser.getBrowserForTab(tab);
                var uri = browser.currentURI.spec; 
                if (uri.match(this.playerConfig.playerURLRegExp)) return true;
            }
        }
    }
    return false;
};

try {
    const NSGetFactory = XPCOMUtils.generateNSGetFactory([FoxyTunesYouTubeController]);
} catch(e) {};
